/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*******************************************************************************/

#include "adv202.h"
#include "utils.h"

#define		MANY_0S						0x00000000L
#define		MANY_AS						0xAAAAAAAAL
#define		MANY_5S						0x55555555L
#define		MANY_FS						0xFFFFFFFFL

//#define DEBUG_IO 1

unsigned long	ADV202_ReadDirect(PETHJPG2K pDevice, unsigned char Register)
{
	unsigned long Data;

	Data = RD(pDevice->PLX9056_BAR2 + ADV202_REGS_OFFSET + Register*4);

	Data = ( (Register <= CMDSTA) || (Register == IADDR) || (Register == IDATA) )
				?  Data : (Data & 0xFFFFL);
#ifdef DEBUG_IO
	printk("ADV212 READ %08x = %08lx\n", Register, Data);
#endif
	return Data;
}

void ADV202_WriteDirect(PETHJPG2K pDevice, unsigned char Register, unsigned long Data)
{
	if (! ((Register <= CMDSTA) || (Register == IADDR) || (Register == IDATA)) )
		Data &= 0xFFFFL;

#ifdef DEBUG_IO
	printk("ADV212 WRITE %08x = %08lx\n", Register, Data);
#endif
	WD( pDevice->PLX9056_BAR2 + ADV202_REGS_OFFSET + Register*4, Data );
}

unsigned long	ADV202_ReadIndirect(PETHJPG2K pDevice, unsigned long Register)
{
	unsigned long Data;

	ADV202_WriteDirect(pDevice,IADDR,Register);
	Data =  ADV202_ReadDirect(pDevice,IDATA);

	return Data;
}

void ADV202_WriteIndirect(PETHJPG2K pDevice, unsigned long Register, unsigned long Data)
{
	ADV202_WriteDirect(pDevice,IADDR,Register);
	ADV202_WriteDirect(pDevice,IDATA,Data);
}


//Exported

void	ADV202_Reset(PETHJPG2K pDevice)
{
	unsigned long	Data = RD( pDevice->PLX9056_BAR2+CPLD_STATUS );

	WD( pDevice->PLX9056_BAR2+CPLD_STATUS, Data & ~CPLD_ADV202_RESET);

	Sleep(1);	//50 usec needed...

	pDevice->DeviceIsRunning = 0;

	WD( pDevice->PLX9056_BAR2+CPLD_STATUS, Data | CPLD_ADV202_RESET);

#if 0
	ADV202_WriteDirect(pDevice,MMODE, IWIDTH_32BIT | IAUTOSTP_32_BIT);
	ADV202_WriteDirect(pDevice,BUSMODE, HWIDTH_32BIT | DWIDTH_32BIT);

	{
	// dump registers
	  int i;

	  printk("direct:\n");
	  for(i=0; i<16; i++) {
	    ADV202_ReadDirect(pDevice,  i);
	  }
	}

	ADV202_WriteDirect(pDevice, 0x0b, 0x12345678);
	ADV202_ReadDirect(pDevice, 0x0b);

	printk("ADV212 PN: %08lx\n", ADV202_ReadIndirect(pDevice,0xFFFF0254 ));
	printk("ADV212 VER: %08lx\n", ADV202_ReadIndirect(pDevice,0xFFFF0250 ));
#endif
}


unsigned char ADV202_Init(PETHJPG2K pDevice, unsigned char bTestFirmwareRAM, int force)
{
	unsigned short i;

	ADV202_WriteDirect(pDevice,PLL_HI, HCLKD);
	if (force) {
	  ADV202_WriteDirect(pDevice,PLL_LO, force);     // x4
	}
	else
	  ADV202_WriteDirect(pDevice,PLL_LO, 0x0004);     // x4
	Sleep(1);//20 usec to let the PLL settle

	/* Initiate a soft reboot */
	ADV202_WriteDirect(pDevice,BOOT, SOFTRST | NO_BOOT_HOST_MODE );

	/* Wait*/
	Sleep(10);

	/* Verify some direct registers reset values */
	if (	(ADV202_ReadDirect(pDevice,EIRQIE) != 0x0000) ||
		(ADV202_ReadDirect(pDevice,EIRQFLG) != (PFTH|DFTH|AFTH|NFTH)) ) {
		return 0;
	}

	if ( 	(ADV202_ReadDirect(pDevice,BUSMODE) != (HWIDTH_16BIT | DWIDTH_16BIT)) ||
		(ADV202_ReadDirect(pDevice,MMODE) != (IWIDTH_16BIT|IAUTOSTP_32_BIT)) ) {
		return 0;
	}

	if ( 	(ADV202_ReadDirect(pDevice,PLL_HI) != 0x0008) &&
		(ADV202_ReadDirect(pDevice,PLL_HI) != 0x0004) &&
		(ADV202_ReadDirect(pDevice,PLL_HI) != 0x0088)) {
		return 0;
	}
#if 0
	if ( 	(ADV202_ReadDirect(pDevice,PLL_LO) != 0x0008) &&
			(ADV202_ReadDirect(pDevice,PLL_LO) != 0x0004) )
		return 0;
#endif

	/* Initialize Host Interface*/
	ADV202_WriteDirect(pDevice,MMODE, IWIDTH_32BIT | IAUTOSTP_32_BIT);
	ADV202_WriteDirect(pDevice,BUSMODE, HWIDTH_32BIT | DWIDTH_32BIT);

	if ( bTestFirmwareRAM )
	{
		/*Test firmware RAM space - 32 bits autostep is enabled*/
		ADV202_WriteDirect(pDevice,IADDR, FIRMWARE_RAM_START);

		for ( i = 0; i < FIRMWARE_SIZE; i++)
			ADV202_WriteDirect(pDevice, IDATA, MANY_FS);

		ADV202_WriteDirect(pDevice,IADDR, FIRMWARE_RAM_START);

		for ( i = 0; i < FIRMWARE_SIZE; i++)
		  if ( ADV202_ReadDirect(pDevice,IDATA) != MANY_FS) {
				return 0;
		  }

		ADV202_WriteDirect(pDevice,IADDR, FIRMWARE_RAM_START);

		for ( i = 0; i < FIRMWARE_SIZE; i++)
			ADV202_WriteDirect(pDevice, IDATA, MANY_0S);

		ADV202_WriteDirect(pDevice,IADDR, FIRMWARE_RAM_START);

		for ( i = 0; i < FIRMWARE_SIZE; i++)
		  if ( ADV202_ReadDirect(pDevice,IDATA) != MANY_0S) {
				return 0;
		  }
	}

	return 1;
}

unsigned char  ADV202_LoadFirmware(PETHJPG2K pDevice, unsigned long * pFirmware,
						unsigned long FirmwareSizeInDwords)
{
	unsigned long i;


	ADV202_WriteDirect(pDevice,IADDR,FIRMWARE_RAM_START);

	for (  i = 0; i < FirmwareSizeInDwords; i++, pFirmware++ )
		//change Endianness
		ADV202_WriteDirect(pDevice,IDATA, SWAP_ULONG(*pFirmware));

	pDevice->DeviceIsBooting = 1;

	/* Initiate a soft reboot */
	ADV202_WriteDirect(pDevice,BOOT, SOFTRST | COPROCESSOR_BOOT );

	Sleep(10);

	/* Initialize Host Interface*/
	ADV202_WriteDirect(pDevice,MMODE, IWIDTH_32BIT | IAUTOSTP_32_BIT);
	ADV202_WriteDirect(pDevice,BUSMODE, HWIDTH_32BIT | DWIDTH_32BIT);

	return 1;
}

unsigned char  ADV202_CheckApplicationID(PETHJPG2K pDevice, unsigned short ID)
{
	unsigned short Timeout = CHECK_ID_TIMEOUT;
	//unsigned long Data;

	//Enable SWIRQ0 external interrupt
	ADV202_WriteDirect(pDevice, EIRQIE, SWIRQ0);

	while ( --Timeout ) {
		if ( ADV202_ReadDirect(pDevice, SWFLAG) == ID )
			break;
		Sleep(1);
	}

	//Disable SWIRQ0 external interrupt
	ADV202_WriteDirect(pDevice, EIRQIE, 0);

	//FW ID must be checked anyway to correctly start FW;
	//if we are not asked to match the FW ID return gracefully
	if (!ID)
		return 1;

	return (unsigned char)Timeout;
}

unsigned char	ADV202_LoadFirmwareParameters(PETHJPG2K pDevice, unsigned char Type,
							PETHJPG2K_FIRMWARE_PARAMS pFirmParams)
{
	unsigned long i;
	ETHJPG2K_VIDEO_ENCODE_FIRMWARE_PARAMS Enc 	= pFirmParams->EncodeParams;
	ETHJPG2K_VIDEO_DECODE_FIRMWARE_PARAMS Dec 	= pFirmParams->DecodeParams;
	ETHJPG2K_HIPI_FIRMWARE_PARAMS			Hipi 	= pFirmParams->HipiParams;

	/*Indirect address auto increment enabled*/

	/*Zero firmware registers*/
	ADV202_WriteDirect(pDevice,IADDR,FIRMWARE_PARAMS);
	for( i = 0; i < 64; i++)
		ADV202_WriteDirect(pDevice,IDATA,0x0L);

	/*Load firmware registers*/
	ADV202_WriteDirect(pDevice,IADDR,FIRMWARE_PARAMS);

	switch(Type)
	{
	case FW_VIDEO_ENCODE:
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Enc.UNI,Enc.XFORMLEV,
								       Enc.PREC,Enc.VFORMAT));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Enc.ATTRTYPE,Enc.STALLPAR,
								       Enc.WKERNEL_QUANT,Enc.CBSIZE));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Enc.RCVAL[2],Enc.RCVAL[1],
								       Enc.RCVAL[0],Enc.RCTYPE));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Enc.COD_STYLE,Enc.QFACT,
								       Enc.PICFG,Enc.J2KPROG));
		break;

	case FW_VIDEO_DECODE:
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Dec.UNI,0,
								       Dec.PREC,Dec.VFORMAT));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(0,0,
								       0,0));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Dec.COD_STYLE,Dec.DRES,
								       Dec.PICFG,0));
		break;

	case FW_HIPI_ENCODE:
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Hipi.sub.EncodeParams.UNI,
								       Hipi.sub.EncodeParams.XFORMLEV,
								       Hipi.sub.EncodeParams.PREC,
								       Hipi.sub.EncodeParams.VFORMAT));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Hipi.sub.EncodeParams.ATTRTYPE,
								       Hipi.sub.EncodeParams.STALLPAR,
								       Hipi.sub.EncodeParams.WKERNEL_QUANT,
								       Hipi.sub.EncodeParams.CBSIZE));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Hipi.sub.EncodeParams.RCVAL[2],
								       Hipi.sub.EncodeParams.RCVAL[1],
								       Hipi.sub.EncodeParams.RCVAL[0],
								       Hipi.sub.EncodeParams.RCTYPE));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Hipi.sub.EncodeParams.COD_STYLE,
								       Hipi.sub.EncodeParams.QFACT,
								       Hipi.sub.EncodeParams.PICFG,
								       Hipi.sub.EncodeParams.J2KPROG));
		ADV202_WriteIndirect(pDevice,  0xFFFF0400, ADV202_REG16(Hipi.PMODE));
		ADV202_WriteIndirect(pDevice,  0xFFFF040C, ADV202_REG16(Hipi.XTOT));
		ADV202_WriteIndirect(pDevice,  0xFFFF0410, ADV202_REG16(Hipi.YTOT));
		//ADV202_WriteIndirect(pDevice,  0xFFFF0414, 0x00010000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0414, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0418, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF041C, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0420, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0424, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0428, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF042C, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0430, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0448, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF044C, 0x00120000);

		break;

	case FW_HIPI_DECODE:
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Hipi.sub.DecodeParams.UNI,
								       0,
								       Hipi.sub.DecodeParams.PREC,
								       Hipi.sub.DecodeParams.VFORMAT));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(0,0,
								       0,0));
		ADV202_WriteDirect(pDevice,IDATA,BYTES_TO_ULONG(Hipi.sub.DecodeParams.COD_STYLE,
								       Hipi.sub.DecodeParams.DRES,
								       Hipi.sub.DecodeParams.PICFG,
								       0));
		ADV202_WriteIndirect(pDevice,  0xFFFF0400, ADV202_REG16(Hipi.PMODE));
		ADV202_WriteIndirect(pDevice,  0xFFFF040C, ADV202_REG16(Hipi.XTOT));
		ADV202_WriteIndirect(pDevice,  0xFFFF0410, ADV202_REG16(Hipi.YTOT));
		ADV202_WriteIndirect(pDevice,  0xFFFF0414, 0x00010000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0418, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF041C, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0420, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0424, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0428, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF042C, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0430, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF0448, 0x00000000);
		ADV202_WriteIndirect(pDevice,  0xFFFF044C, 0x00100000);

		break;


	default:
		return 0;
	}

	return 1;
}

void	ADV202_SetupDMA(PETHJPG2K pDevice)
{
	unsigned char FwType = pDevice->FirmwareType;

#if 1
	// set threshold
#define THRESHOLD 128
	ADV202_WriteIndirect(pDevice, FFTHRC, ADV202_REG16(THRESHOLD));
	ADV202_WriteIndirect(pDevice, FFTHRP, ADV202_REG16(THRESHOLD));
#undef THRESHOLD
#endif

	switch(FwType)
	{
		case FW_VIDEO_ENCODE:
			//Compressed data output on DMA 0
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_CODE));
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_CODE|DMEN));
		break;

		case FW_VIDEO_DECODE:
			//Compressed data input on DMA 1
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_CODE));
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_CODE|DMEN));
		break;

		case FW_HIPI_LOOP:
			//data output on DMA 0
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_ATTR));
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_ATTR|DMEN));
			//data input on DMA 1
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_CODE));
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_CODE|DMEN));
		break;

		case FW_HIPI_ENCODE:
			// compressed data output on DMA 0
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_CODE));
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_CODE|DMEN));
			// pixel data input on DMA 1
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_PIXEL));
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_PIXEL|DMEN));
		break;

		case FW_HIPI_DECODE:
			// pixel output on DMA 0
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_PIXEL));
			ADV202_WriteIndirect(pDevice, EDMOD0, ADV202_REG16(DMSEL_PIXEL|DMEN));
			// compressed input on DMA 1
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_CODE));
			ADV202_WriteIndirect(pDevice, EDMOD1, ADV202_REG16(DMSEL_CODE|DMEN));
		break;

		default:;
	}
}

void	ADV202_StartExecution(PETHJPG2K pDevice)
{
	//unsigned long Data = ADV202_ReadDirect(pDevice, EIRQFLG);

	/*Clear SWIRQ0*/
	ADV202_WriteDirect(pDevice, EIRQFLG, 0xffff);
#if 0
	{
	// dump registers
	  int i;

	  printk("direct:\n");
	  for(i=0; i<16; i++) {
	    printk("%09x = %08lx\n", i, ADV202_ReadDirect(pDevice,  i));
	  }

	  printk("indirect:\n");
	  for(i=0xffff0400; i<0xffff0450; i+=4) {
	    printk("%09x = %08lx\n", i, ADV202_ReadIndirect(pDevice,  i));
	  }
	  for(i=0xffff1408; i<0xffff1424; i+=4) {
	    printk("%09x = %08lx\n", i, ADV202_ReadIndirect(pDevice,  i));
	  }

	  printk("parameters:\n");
	  for(i=FIRMWARE_PARAMS; i<(FIRMWARE_PARAMS+64); i+=4) {
	    printk("%09x = %08lx\n", i, ADV202_ReadIndirect(pDevice,  i));
	  }
	}
#endif
}

void ADV202_Stop(PETHJPG2K pDevice)
{
	ADV202_Reset(pDevice);
}

void ADV202_SetQ(PETHJPG2K pDevice, int q)
{
	unsigned long Data = ADV202_ReadIndirect(pDevice,  0x00057F08);

	Data = BYTES_TO_ULONG((q >>  0) & 0xff,
			      (q >>  8) & 0xff,
			      (q >> 16) & 0xff,
			      (Data >> 24) & 0xff);
	ADV202_WriteIndirect(pDevice, 0x00057F08, Data);
}




