/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

#ifndef _ADV202_H
#define _ADV202_H
#include "driver.h"
#include "ethjpg2k.h"
#include "adv202_hdr.h"

/*
CPLD Internal Registers
-----------------------
*/

#define	CPLD_STATUS				0x20
// GPIO offset in CPLD
#define GPIO_OUT        0x24
#define GPIO_IN         0x28

#define	CPLD_ADV202_RESET			0x01		// 0 <= ADV202 in reset, 1 <= ADV202 working
#define	CPLD_TWA2835_RESET			0x02		// 0 <= TWA in reset, 1 <= TWA working
#define	CPLD_ADV202_IRQ			        0x04	        // 0 <= ADV202 irq off, 1 <= ADV202 irq on
#define	CPLD_TWA2835_IRQ			0x08		// 0 <= TWA irq off , 1 <= TWA irq on


/*
Direct Register Definition
--------------------------
The host has to initialize these registers before any application specific operation can take place.
Registers width is 16/32 bits
*/
#ifdef ANALOG_DEMO
#define   ADV202_REGS_OFFSET		0x0
#else
#define   ADV202_REGS_OFFSET		0x80		/*ADV202 registers offset in CPLD*/
#endif

#define   PIXEL     0x00         /*Pixel FIFO Access Register                   */
#define   CODE      0x01         /*Compressed Code Stream Access Register       */
#define   ATTR      0x02         /*Attribute FIFO Access Register               */
#define   ANCL      0x03         /*Ancillary FIFO Access Register               */
#define   CMDSTA    0x04         /*Command Stream FIFO                          */
#define   EIRQIE    0x05         /*External Interrupt Enabled                   */
#define   EIRQFLG   0x06         /*External interrupt Flags                     */
#define   SWFLAG    0x07         /*Software Flag Register                       */
#define   BUSMODE   0x08         /*Bus Mode Configuration Register              */
#define   MMODE     0x09         /*Miscellaneous Mode Register                  */
#define   STAGE     0x0A         /*Staging Register                             */
#define   IADDR     0x0B         /*Indirect Address Register                    */
#define   IDATA     0x0C         /*Indirect Data Register                       */
#define   BOOT      0x0D         /*Boot Mode Register                           */
#define   PLL_HI    0x0E         /*PLL Control Register - High Byte             */
#define   PLL_LO    0X0F	  		/*PLL Control Register - Low Byte              */

/*Bitfields*/
/*EIRQIE & EIRQFLG*/
#define PFTH        0x0001        /*Pixel FIFO threshold condition                  */
#define DFTH        0x0002        /*Data FIFO threshold condition                   */
#define AFTH        0x0004        /*Attribute FIFO threshold condition              */
#define NFTH        0x0008        /*Ancillary FIFO threshold condition              */
#define PFERR       0x0010        /*Pixel FIFO overflow/underflow                   */
#define DFERR       0x0020        /*Data FIFO overflow/underflow                    */
#define AFERR       0x0040        /*Attribute FIFO overflow/underflow               */
#define NFERR       0x0080        /*Ancillary FIFO overflow/underflow               */
#define SWIRQ0      0x0400        /*Software interrupt 0                            */
#define SWIRQ1      0x0800        /*Software interrupt 1                            */
#define SWIRQ2      0x1000        /*Software interrupt 2 (reserved for future use)  */
#define INDERR      0x2000        /*Indirect register access error                  */
#define IHWDIRQ14   0x4000        /*Internal hardware interrupt                     */
#define FERR        0x8000        /*ADV202 fatal error                              */

/*BUSMODE*/
#define HWIDTH_16BIT     0x01		/*Host control data width*/
#define HWIDTH_32BIT     0x02
#define DWIDTH_16BIT     0x04		/*DMA data width*/
#define DWIDTH_32BIT     0x08
#define BCFG_NORM        0x00		/*Bus configuration*/
#define BCFG_JDATA       0x10
#define BCFG_DLI         0x20

/*MMODE*/
#define IWIDTH_16BIT     0x01		/*Indirect access width*/
#define IWIDTH_32BIT     0x02
#define IAUTOSTP_8BIT    0x00   	/*IADDR increment/decrement size*/
#define IAUTOSTP_16BIT   0x04
#define IAUTOSTP_32_BIT  0x08
#define IAUTOSTP_NONE    0x0C
#define IAUTOMOD_INC     0x00   	/*Indirect address auto increment/decrement*/
#define IAUTOMOD_DEC     0x10
#define CTLREGAM_FULL    0x00   	/*Control register address mode*/
#define CTLREGAM_CTRL    0x20

/*BOOT*/
/*BIT3 is reserved and must always be written to 1*/
#define NO_BOOT_HOST_MODE     (0x02 | 0x08)
#define SOC_BOOT              (0x04 | 0x08)
#define COPROCESSOR_BOOT      (0x05 | 0x08)
#define BOOT_OVER_SPI         (0x06 | 0x08)
#define HARDRST               (0x40 | 0x08)
#define SOFTRST               (0x80 | 0x08)

/*PLL_HI*/
#define PLLPDN                0x01 /*Power down enable */
#define BYPASS                0x02 /*Bypass enabled */
#define HCLKD                 0x08 /*HCLK divider enabled */
#define ACK_FN                0x10 /*0 <= ACK/ is in an output driver (drives 1 or 0)*/
                                   /*1 <= ACK/ is configured as an open drain output*/

/*PLL_LO*/
/*bit [0..4] <= multiplier [0...31]*/
#define LFB                   0x20 /*Loop feedback divider enable */
#define IPD                   0x80 /*Input clock divider enable */




/*
Indirect Register Definition
-------------------------
These registers are generally accessed by firmware only.
In certain modes, for instance custom specific input format or HIPI mode, indirect registers have to be
accessed by the user through the use of the IADDR and IDATA registers. The indirect register address
space starts at internal address 0xFFFF0000.
If the CTRLREGAM bit in the direct register MMODE is set to FULL ADDRESS MODE, then the complete
address must be loaded into IADDR before accessing the desired indirect register. Otherwise, the
user only needs to load the 16 least significant bits of the desired register address.
Registers width is 16 bits.
*/

#define  PMODE1              0xFFFF0400L    /*Pixel/Video Format                 */
#define  COMP_CNT_STATUS     0xFFFF0404L    /*Horizontal count                   */
#define  LINE_CNT_STATUS     0xFFFF0408L    /*Vertical count                     */
#define  X_TOT                0xFFFF040CL    /*Total Samples per line             */
#define  Y_TOT                0xFFFF0410L    /*Total Lines per frame              */
#define  F0_START            0xFFFF0414L    /*Start Line of Field 0 [F0]         */
#define  F1_START            0xFFFF0418L    /*Start Line of Field 1 [F1]         */
#define  V0_START            0xFFFF041CL    /*Start of active video Field 0 [F0] */
#define  V1_START            0xFFFF0420L    /*Start of active video Field 1 [F1] */
#define  V0_END              0xFFFF0424L    /*End of active video Field 0 [F0]   */
#define  V1_END              0xFFFF0428L    /*End of active video Field 1 [F1]   */
#define  PIXEL_START         0xFFFF042CL    /*Horizontal start of active video   */
#define  PIXEL_END           0xFFFF0430L    /*Horizontal end of active video     */
#define  PI_STATUS           0xFFFF0434L    /*Pixel/Video Status Register        */
#define  MS_CNT_DEL          0xFFFF0440L    /*Master/Slave Delay                 */
#define  LINE_CNT_INTERRUPT  0xFFFF0444L    /*Line count interrupt               */
#define  PMODE2              0xFFFF0448L    /*Pixel Mode 2                       */
#define  VMODE               0xFFFF044CL    /*Video Mode                         */
#define  EDMOD0              0xFFFF1408L    /*External DMA mode register 0       */
#define  EDMOD1              0xFFFF140CL    /*External DMA mode register 1       */
#define  FFMODE              0xFFFF1418L    /*FIFO mode register                 */
#define  FFTHRP              0xFFFF1410L    /*FIFO Threshold for Pixel FIFO      */
#define  FFTHRC              0xFFFF141CL    /*FIFO Threshold for Code FIFO       */
#define  FFTHRA FIFO         0xFFFF1420L    /*Threshold for ATTR FIFO            */
#define  FFTHRN FIFO         0xFFFF1424L    /*Threshold for ANCL FIFO            */
#define  FFCNTP FIFO         0xFFFF1414L    /*Full/Empty count for PIXEL FIFO    */
#define  FFCNTC FIFO         0xFFFF1428L    /*Full/ Empty count for CODE FIFO    */
#define  FFCNTA FIFO         0xFFFF142CL    /*Full/Empty count for ATTR FIFO     */
#define  FFCNTN FIFO         0xFFFF1430L    /*Full/Empty count for ANCL FIFO     */

/*Indirect registers values
Except where noted values are NOT bit field values
*/

/*PMODE1*/
#define	PFMT_VDATA_SINGLE                  0x04 /*Single Component (8, 10, or 12 bit)                  */
#define	PFMT_VDATA_CBYCRY_INTERL           0x05 /*Cb/Y/Cr/Y interleaved (8, 10, or 12 bit)             */
#define	PFMT_VDATA_CBCR_INTERL             0x07 /*Cb/Cr interleaved (8, 10, or 12 bit)                 */
#define	PFMT_VDATA_2X12_PACK_SINGLE        0x10 /*24 bit 2x12-bit Packed Single Component (12 bit)     */
#define	PFMT_VDATA_2X12_PACK_CBCR_INTERL   0x11 /*24-bit 2x12-bit Packed CbY/CrY interleaved (12 bit)  */
#define	PFMT_VDATA_2X12_PACK_CBCR          0x13 /*24-bit 2x12-bit Packed CbCr (12 bit)                 */
#define	PFMT_HDATA_4X8_SINGLE              0x14 /*32-bit 4x8-bit Single Component            */
#define	PFMT_HDATA_4X8_PACK_YCBCR          0x15 /*32-bit 4x8-bit Packed YCbYCr               */
#define	PFMT_HDATA_4X8_PACK_YYCBCR         0x16 /*32-bit 4x8-bit Packed YYCbCr               */
#define	PFMT_HDATA_4X8_PACK_CBCRCBCR       0x17 /*32-bit 4x8-bit Packed CbCrCbCr             */
#define	PFMT_HDATA_2X16_PACK_SINGLE        0x18 /*32-bit 2x16-bit Packed Single Component    */
#define	PFMT_HDATA_2X16_PACK_YCBYCR        0x19 /*32-bit 2x16-bit Packed YCb/YCr             */
#define	PFMT_HDATA_2X16_PACK_YYCBCR        0x1A /*32-bit 2x16-bit Packed YY/CbCr             */
#define	PFMT_HDATA_2X16_PACK_CBCR          0x1B /*32-bit 2x16-bit Packed CbCr                */
#define	PFMT_PREC_8                        0x000     /*8-bit precision    */
#define	PFMT_PREC_10                       0x100     /*10-bit precision   */
#define	PFMT_PREC_12                       0x200     /*12-bit precision   */
#define	PFMT_PREC_14                       0x300     /*14-bit precision   */
#define	PFMT_PREC_16                       0x400     /*16-bit precision   */


/*PMODE2*/
/*Bitfield*/
#define   VCLK_POL  0x01 /*VCLK active edge 1=rising, 0=falling*/
#define   VSYNC_POL 0x02 /*VSYNC active edge 1=rising, 0=falling*/
#define   HSYNC_POL 0x04 /*HSYNC active edge 1=rising, 0=falling*/
#define   FIELD_POL 0x08 /*FIELD active edge 1=rising, 0=falling*/
#define   YUNI      0x10 /*Y Unipolar 1=on, 0=off
                         For 8-bit pixel samples (for example), Unipolar
                         defines a range from 0 to 255. When this bit is disabled
                         the range is defined to be -127 to +127.*/
#define   CUNI      0x20 /*C Unipolar 1=on, 0=off
                         For 8-bit pixel samples (for example), Unipolar
                         defines a range from 0 to 255. When this bit is disabled
                         the range is defined to be -127 to +127*/
/*VMODE*/
/*Bitfield*/
#define   MAS_SLV   	0x01 		/*Master or Slave operation is only selectable in Decode
										mode. 1=MASTER 0=SLAVE*/
#define   ENC_DEC   	0x02 		 /*Sets ENCODE or DECODE mode 1=ENCODE 0=DECODE*/
#define   MP_656    	0x04 		 /*Video input timing control.
								 		0=EAV/SAV mode = 0; for video input with embedded timing codes
								  		1=HVF mode = 1; for video input with separate H,V,F sync signals.*/
#define   DUAL LANE 	0x08 		 /*Dual Lane mode.Used to enable a dual-lane video interface to handle
								 		up to two 12-bit wide samples on the VDATA/XGPIO pins.
								 		1=Dual-Lane Mode on 0=Dual-Lane Mode off*/
#define   HOST_MODE 	0x10 		  /*Pixel data over HDATA bus. 1=on 0=off*/
#define   RAW_MODE  	0x20 		 /*Raw video input on the VDATA bus. HVF mode is not required
								 		to be set in this mode. 1=on 0=off*/
#define   PRGRSV_SCN 0x08 		/*Progressive scan mode; Must be set if video input on the VDATA
								  		bus is non-interlaced, i.e 1field/frame. 1=on 0=off*/
#define   CNT_RD_EN	0x800		/*Read PI control counters [CO_CNT_STATUS and
								 		LINE_CNT_STATUS]. 1=on 0=off*/

/*EDMOD0 & EDMOD1: DMA 0/1 management*/
/*Bitfield*/
#define   DMEN          0x01       /*Enable DMA channel. 0=disabled 1=enabled*/
#define   DMSEL_PIXEL   0x00       /*DMA Channel Assignment*/
#define   DMSEL_CODE    0x02
#define   DMSEL_ATTR    0x04
#define   DMSEL_ANCL    0x06
											  /*bits[5..3]  DMA Channel mode*/
#define   DMMOD_DED               0x00      /*Dedicated Chip Select DMA mode*/
#define   DMMOD_SINGLE            (0x01<<3) /*Single transfer DREQ/DACK DMA mode*/
#define   DMMOD_BURST             (0x02<<3) /*Burst transfer DREQ/DACK DMA mode*/
#define   DMMOD_JDATA             (0x03<<3) /*JDATA mode*/
#define   DMMOD_SINGLE_FLY_BY     (0x05<<3) /*Single transfer Fly-by DMA mode*/
#define   DMMOD_BURST_FLY_BY      (0x06<<3) /*Burst transfer Fly-by DMA mode*/
											/*bits[8..6] DMA Channel burst length [in number of accesses]*/
#define   DMBL_8                  (0x00<<6) /*8 - not available for DWIDTH of 8-bit      */
#define   DMBL_16                 (0x01<<6) /*16 - for DWIDTH of 8/16 or 32-bits         */
#define   DMBL_32                 (0x02<<6) /*32 - for DWIDTH of 8/16 or 32-bits         */
#define   DMBL_64                 (0x03<<6) /*64 - for DWIDTH of 8/16 or 32-bits         */
#define   DMBL_128                (0x04<<6) /*128 - for DWIDTH of 8/16 or 32-bits        */
#define   DMBL_512                (0x06<<6) /*512 - for DWIDTH of 8/16-bit, for 32-bits not recommended*/
#define   DMBL_1024               (0x07<<6) /*1024 - for DWIDTH of 8-bit, for 16-bit not recommended,
                                             for 32-bits not available.*/
#define   DRPOL    0x200     /*FSRQ / VALID polarity. 0 =Active Low, 1 = Active High*/
#define   DAPOL    0x400     /*DAPOL FCS / HOLD polarity. 0 =Active Low, 1 = Active High*/
#define   DRPULS   0x800     /*bits[14..11]:  DREQ pulse width [in Jclkcycles]
                              If DRPULS==0, then DREQ will remain asserted until
                              DACK and RD/ in read mode or WR/ in write mode are asserted.*/

/*FFMODE*/
/*Bits[3..0] Enables and sets the sizes of the FIFOs associated with the
CODE, ATTR and ANCL data channels. The maximum
data width of these FIFOs is 32-bits.*/
								 /*CODE	  ATTR ANCL*/
#define   FFCFG_0   0    /*256     256  N/A */
#define   FFCFG_1   1    /*256     128  128 */
#define   FFCFG_2   2    /*384     128  N/A */
#define   FFCFG_3   3    /*384     64   64  */
#define   FFCFG_4   4    /*512     N/A  N/A */
#define   CINP      0x10 /*Sets the CODE data FIFO as an input channel*/
#define   AINP      0x20 /*Sets the ATTR data FIFO as an input channel*/
#define   NINP      0x40 /*Sets the ANCL data FIFO as an input channel*/




/*Firmware*/
#define	FIRMWARE_RAM_START	0x50000L
#define	FIRMWARE_SIZE	   	0x2000    		/*In 32-bits units => 32KB*/
#define	ENCODE_FIRMWARE_ID	0xFF82
#define	DECODE_FIRMWARE_ID	0xFFA2
#define	DMATEST_FIRMWARE_ID	0xFF04

#define	FIRMWARE_PARAMS		0x57F00L

/*Encode firmware parameters ( offset from FIRMWARE_PARAMS )
Except where noted registers are byte wide and values are NOT bit field values*/
#define   ENC_VFORMAT             0x00 /*Video Standard                   */
#define   ENC_VFORMAT_NTSC        0    /*NTSC 4:2:2                       */
#define   ENC_VFORMAT_PAL         1    /*PAL 4:2:2                        */
#define   ENC_VFORMAT_1080i_LUM   2    /*1080i Luminance                  */
#define   ENC_VFORMAT_1080i_CHR   3    /*1080i Chrominance                */
#define   ENC_VFORMAT_CUSTOM      4    /*Custom [refer to chapter 8]      */
#define   ENC_VFORMAT_720_60_LUM  5    /*720/60p Luminance                */
#define   ENC_VFORMAT_720_60_CHR  6    /*720/60p Chrominance              */
#define   ENC_VFORMAT_NTSC_DEI    8    /*NTSC 4:2:2 De-interlace          */
#define   ENC_VFORMAT_PAL_DEI     9    /*PAL 4:2:2 De-interlace           */

#define   ENC_PREC                0x01 /*Precision    */
#define   ENC_PREC_8              0    /*8 bit        */
#define   ENC_PREC_10             1    /*10-bit       */
#define   ENC_PREC_12             2    /*12-bit       */

#define   ENC_XFORMLEV            0x2  /*Number of Wavelet Transform Levels (1-6)   */

#define   ENC_UNI                 0x3  /*Component polarity          */
#define   ENC_UNI_BC_BY           0    /*Bipolar C, Bipolar Y        */
#define   ENC_UNI_UC_BY           1    /*Unipolar C, Bipolar Y       */
#define   ENC_UNI_BC_UY           2    /*Bipolar C, Unipolar Y       */
#define   ENC_UNI_UC_UY           3    /*Unipolar C, Unipolar Y      */

#define   ENC_CBSIZE              0x4  /*Codeblock Dimensions                       */
#define   ENC_CBSIZE_32X32        0    /*32x32 (will not work for 1080i or 720p)    */
#define   ENC_CBSIZE_64X32        1    /*64x32                                      */
#define   ENC_CBSIZE_64X64        2    /*64x64                                      */
#define   ENC_CBSIZE_128x32       3    /*128x32                                     */

#define   ENC_WAVELET_KERNEL_QUANT       0x5  /*Wavelet Kernel and Quantization factor     */
#define   ENC_WAVELET_KERNEL_QUANT0_I9_7 0    /*Irreversible 9/7 using fixed table [lossy]
												 - typically provides the most compression  */
#define   ENC_WAVELET_KERNEL_QUANT0_R5_3 1    /*Reversible 5/3 [lossless]                  */
#define   ENC_WAVELET_KERNEL_QUANT0_I5_3 2    /*Irreversible 5/3 [lossy]                   */

#define   ENC_STALLPAR            0x6  /*Skip Fields in Encode mode       */
#define   ENC_STALLPAR_NONE       0    /*capture all fields               */
/*#define      1-63           skip this number of fields              */
#define   ENC_STALLPAR_STOP       0x80 /*stop capture                     */

#define   ENC_ATTRTYPE            0x7  /*Attribute Data output format (this feature will only work for RCTYPE=0) */
#define   ENC_ATTRTYPE_NONE       0    /*Do not output attribute data.                                           */
#define   ENC_ATTRTYPE_ADV        1    /*Use ADV-JP2000 attribute data format                                    */
#define   ENC_ATTRTYPE_CH_WO_ADJ  2    /*Log2 Convex Hull without Adjustments                                    */
#define   ENC_ATTRTYPE_CH_W_ADJ   3    /*Log2 Convex Hull with Quantization Adjust                               */
#define   ENC_ATTRTYPE_FLD        4    /*ADV202 field header output to ATTR FIFO                                 */
#define   ENC_ATTRTYPE_FLD_PCK    5    /*ADV202 field header and packet byte length output to ATTR FIFO          */

#define   ENC_RCTYPE              0x8  /*RCTYPE Rate Control Algorithm                                           */
#define   ENC_RCTYPE_NONE         0    /*None (no truncation)                                                    */
#define   ENC_RCTYPE_SIZE         1    /*Target Size per video field/image within 5% of RCVAL.                   */
#define   ENC_RCTYPE_QUAL         2    /*Target Quality per video field/image.                                   */
#define   ENC_RCTYPE_ML_SIZE      3    /*Multi-layered Target Size per video field/image.
                                    Requires RCVAL and LTARGET parameters also.                               */
#define   ENC_RCTYPE_ML_QUAL      4    /*Multi-layered Target Quality per video field/ image.
                                   Requires RCVAL and LTARGET parameters also.                               */

#define   ENC_RCVAL               0x9  /*Target Size or Quality factor.                                          */
                                   /*If RCTYPE = 0 then RCVAL = n/a                                          */
                                   /*If RCTYPE = 1 then RCVAL= Size of field/image in bytes                  */
                                   /*If RCTYPE = 2 then RCVAL = Quality factor.                              */
                                   /*If RCTYPE >= 3 then RCVAL = Number of layers.                           */
                                   /*==> This register is 2*19  bytes wide <==                               */

#define   ENC_J2KPROG             0xC  /*JPEG2000 Progression Style*/
#define   ENC_J2KPROG_LRCP        0
#define   ENC_J2KPROG_RLCP        1
#define   ENC_J2KPROG_RPCL        2
#define   ENC_J2KPROG_PCRL        3
#define   ENC_J2KPROG_CPRL        4

#define   ENC_PICFG               0xD  /*Pixel Interface Configuration Settings:                             */
/*==> Bitfields<==*/
#define   ENC_PICFG_VCLK_POL      0x01 /*VCLK Polarity  (1=rising, 0=falling)                                */
#define   ENC_PICFG_VSYNC_POL     0x02 /*VSYNC Polarity (1=rising, 0=falling)                                */
#define   ENC_PICFG_HSYNC_POL     0x04 /*HSYNC Polarity (1=rising, 0=falling)                                */
#define   ENC_PICFG_FIELD_POL     0x08 /*Field Polarity (1=rising, 0=falling)                                */
#define   ENC_PICFG_HVF_PINS      0x40 /*Use HVF pins   (1=use external HVF pins 0=use embedded SAV/EAV)     */

#define   ENC_QFACT               0xE  /*Quantization factor         */
                                   /*255 = 255/256 factor        */
                                   /*1 = 1/256 factor            */
                                   /*0 = 256/256 factor (1x)     */

#define   ENC_COD_STYLE           0xF  /*Output Code Stream Format   */
/*???==> Bitfields<== ???                                        */
#define   ENC_COD_STYLE_ADV       0    /*ADV202 Raw Format           */
#define   ENC_COD_STYLE_J2C       1    /*J2C Format                  */
#define   ENC_COD_STYLE_JP2       2    /*JP2 Format                  */
/*The following applies to J2C or JP2 Formats only               */
/*==> Bitfields<==*/
#define   ENC_COD_STYLE_PPT       0x80 /*0 = Packet headers with packet body, 1 = use PPT     */
#define   ENC_COD_STYLE_PLT       0x40 /*0 = no PLT, 1 = include PLT                          */
#define   ENC_COD_STYLE_SOP       0x20 /*0 = no SOP, 1 = include SOP                          */
#define   ENC_COD_STYLE_EPH       0x10 /*0 = no EPH, 1 = include EPH                          */

#define   ENC_STEPSIZES           0x10
/*
Quantization Stepsizes using 5 exponent bits plus 11 mantissa bits.
For more detail refer to the ISO/IEC15444-1 specification, section Annex E.
There is one step size assigned per subband (note there is only one set of stepsizes defined
which all components use).
The order of subband stepsizes for this parameter list is as follows for 6 transform levels:
LL
1HL (for > 0 transform levels)
1LH (for > 0 transform levels)
1HH (for > 0 transform levels)
2HL (for > 1 transform levels)
2LH (for > 1 transform levels)
2HH (for > 1 transform levels)
3HL (for > 2 transform levels)
3LH (for > 2 transform levels)
3HH (for > 2 transform levels)
4HL (for > 3 transform levels)
4LH (for > 3 transform levels)
4HH (for > 3 transform levels)
5HL (for > 4 transform levels)
5LH (for > 4 transform levels)
5HH (for > 4 transform levels)
6HL (for > 5 transform levels)
6LH (for > 5 transform levels)
6HH (for > 5 transform levels)
==> This register is 2*19  bytes wide <==*/

#define   ENC_LOAD_SS             0x36
/*
Bit 0 of this byte MUST BE '0' prior to loading the stepsizes and/or QFACT.
After writing the stepsizes and/or QFACT, the user must set Bit 0 to '1' to tell the firmware
that the stepsizes/QFACT are valid and ready for use.
Once the firmware loads these new stepsizes/QFACT, this bit will again be reset to '0'.
*/

#define   ENC_LOAD_VW             0x37 /*Visual Weighting Options                                                                */
#define   ENC_LOAD_VW_NONE_PREV   0    /*No visual weighting or use previously loaded visual weights.                            */
#define   ENC_LOAD_VW_CUSTOM      1    /*Load custom visual weights using VW_Y, VW_CB and VW_CR parameters.                      */
#define   ENC_LOAD_VW_RECC        2    /*Load visual weights as recommended in ISO/IEC 15444-3 standard for Motion Jpeg2000      */

#define   ENC_VW_Y                0x38 /*Visual Weighting Factors for Luminance.
                                   ==> This register is 2*19  bytes wide <==    */
#define   ENC_VW_CB               0x5E /*Visual Weighting Factors for Cb.
                                   ==> This register is 2*19  bytes wide <==    */
#define   ENC_VW_CR               0x84 /*Visual Weighting Factors for Cr.
                                   ==> This register is 2*19  bytes wide <==    */
#define   ENC_RESERVED            0xAA /*Reserved Write 0x00h.
                                   ==> This register is 6     bytes wide <==    */
#define   ENC_LTARGET             0xB0 /*Target for each layer.
                                   ==> This register is 4*16  bytes wide <==    */

/*Decode firmware parameters ( offset from FIRMWARE_PARAMS )
Except where noted registers are byte wide and values are NOT bit field values*/
#define   DEC_VFORMAT             0x00 /*Video Standard                   */
#define   DEC_VFORMAT_NTSC        0    /*NTSC 4:2:2                       */
#define   DEC_VFORMAT_PAL         1    /*PAL 4:2:2                        */
#define   DEC_VFORMAT_1080i_LUM   2    /*1080i Luminance                  */
#define   DEC_VFORMAT_1080i_CHR   3    /*1080i Chrominance                */
#define   DEC_VFORMAT_CUSTOM      4    /*Custom [refer to chapter 8]      */
#define   DEC_VFORMAT_720_60_LUM  5    /*720/60p Luminance                */
#define   DEC_VFORMAT_720_60_CHR  6    /*720/60p Chrominance              */
#define   DEC_VFORMAT_NTSC_DEI    8    /*NTSC 4:2:2 De-interlace          */
#define   DEC_VFORMAT_PAL_DEI     9    /*PAL 4:2:2 De-interlace           */

#define   DEC_PREC                0x01 /*Precision    */
#define   DEC_PREC_8              0    /*8 bit        */
#define   DEC_PREC_10             1    /*10-bit       */
#define   DEC_PREC_12             2    /*12-bit       */

#define   DEC_UNI                 0x3  /*Component polarity          */
#define   DEC_UNI_BC_BY           0    /*Bipolar C, Bipolar Y        */
#define   DEC_UNI_UC_BY           1    /*Unipolar C, Bipolar Y       */
#define   DEC_UNI_BC_UY           2    /*Bipolar C, Unipolar Y       */
#define   DEC_UNI_UC_UY           3    /*Unipolar C, Unipolar Y      */

#define   DEC_PICFG               0x9  /*Pixel Interface Configuration Settings                              */
/*==> Bitfields<==*/
#define   DEC_PICFG_VCLK_POL      0x01 /*VCLK Polarity  (1=rising, 0=falling)                                */
#define   DEC_PICFG_VSYNC_POL     0x02 /*VSYNC Polarity (1=rising, 0=falling)                                */
#define   DEC_PICFG_HSYNC_POL     0x04 /*HSYNC Polarity (1=rising, 0=falling)                                */
#define   DEC_PICFG_FIELD_POL     0x08 /*Field Polarity (1=rising, 0=falling)                                */
#define   DEC_PICFG_MASTER        0x10 /*Master mode    (1=master, 0=slave)                                  */
#define   DEC_PICFG_HVF_PINS      0x40 /*Use HVF pins   (1=use external HVF pins 0=use embedded SAV/EAV)     */


#define   DEC_DRES                0x0A /*Decode Resolution settings*/
/*==> Bitfields<==*/
#define   DEC_DRES_SCALE          0x80 /*Scale Image Size Enable (0 = keep image size the same, 1 = scale image
                                        down to desired resolution size)*/
#define   DEC_DRES_RES_ALL        0    /*Decode all resolutions       */
#define   DEC_DRES_RES_1_4        1    /*Decode to 1/4 resolution     */
#define   DEC_DRES_RES_1_16       2    /*Decode to 1/16 resolution    */
#define   DEC_DRES_RES_1_64       3    /*Decode to 1/64 resolution    */
#define   DEC_DRES_RES_1_256      4    /*Decode to 1/256 resolution   */
#define   DEC_DRES_RES_1_1024     5    /*Decode to 1/1024 resolution  */
#define   DEC_DRES_RES_1_4096     6    /*Decode to 1/4096 resolution  */

#define   DEC_COD_STYLE           0x0B /*Input Code Stream Format    */
#define   DEC_COD_STYLE_ADV       0    /*ADV202 Raw Format           */
#define   DEC_COD_STYLE_J2C       1    /*J2C Format                  */
#define   DEC_COD_STYLE_JP2       2    /*JP2 Format                  */

/*Miscellaneous*/
#define	CHECK_ID_TIMEOUT	0xFF



unsigned long	ADV202_ReadDirect(PETHJPG2K pDevice, unsigned char Register);
void		ADV202_WriteDirect(PETHJPG2K pDevice, unsigned char Register, unsigned long Data);
unsigned long	ADV202_ReadIndirect(PETHJPG2K pDevice, unsigned long Register);
void		ADV202_WriteIndirect(PETHJPG2K pDevice, unsigned long Register, unsigned long Data);
void		ADV202_Reset(PETHJPG2K pDevice);
unsigned char 	ADV202_Init(PETHJPG2K pDevice, unsigned char bTestFirmwareRAM, int force);
unsigned char  	ADV202_LoadFirmware(PETHJPG2K pDevice, unsigned long * pFirmware,
					unsigned long FirmwareSizeInDwords);
unsigned char	ADV202_LoadFirmwareParameters(PETHJPG2K pDevice, unsigned char FwType,
						PETHJPG2K_FIRMWARE_PARAMS pFirmParams);
void		ADV202_SetupDMA(PETHJPG2K pDevice);
void		ADV202_StartExecution(PETHJPG2K pDevice);
unsigned char  	ADV202_CheckApplicationID(PETHJPG2K pDevice,  unsigned short ID);
void ADV202_Stop(PETHJPG2K pDevice);
void ADV202_SetQ(PETHJPG2K pDevice, int q);


#endif	//_ADV202_H

