/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
******************************************************************************/
#ifndef ETHJPG2K_H
#define ETHJPG2K_H
#include "plat.h"

// IOCTLs

/*! The firmware is transferred to the driver via a struct ETHJPG2K_FIRMWARE_DATA. */
#define	IOCTL_ETHJPG2K_ENCDEC_LOAD_FIRMWARE		        IOCTL_ETHJPG2K(0x51)
/*! The encoder/decoder is started. */
#define	IOCTL_ETHJPG2K_ENCDEC_START				IOCTL_ETHJPG2K(0x52)
/*! The encoder/decoder is stopped. */
#define	IOCTL_ETHJPG2K_ENCDEC_STOP				IOCTL_ETHJPG2K(0x53)
/*! Dynamically (so only when the encoder is running) sets the quality
of the output stream. Parameter is an int which is the RCVAL. */
#define	IOCTL_ETHJPG2K_ENC_QUALITY				IOCTL_ETHJPG2K(0x58)

/* gipo stuff */

/*! Reads the current output GPIO value. The argument is a pointer to
an int. This IOCTL is used for masking only the bits that the user
needs to change. */
#define IOCTL_ETHJPG2K_GPIO_READ				IOCTL_ETHJPG2K(0x60)
/*! Writes to he output GPIO pins implemented on the CTR1474. The
argument is a pointer to an int.Only the pins 0-7 are user
available. Bits 8 and 9 control the 2 LEDs on the board. Bit 9 switches
the data path between the video front-end and the ADV212 between
encoding and decoding modes. */
#define IOCTL_ETHJPG2K_GPIO_WRITE				IOCTL_ETHJPG2K(0x61)
/*! Reads the current output GPIO value. The argument is a pointer to
an int. This IOCTL is used for masking only the bits that the user
needs to change. */
#define IOCTL_ETHJPG2K_GPIO_WRITE_READ                          IOCTL_ETHJPG2K(0x62)

/* linux specific ioctls */

/*! This IOCTL initializes the companion chips on the CTR1474. Should be
called before any attempt to encode/decode a video stream. */
#define IOCTL_ETHJPG2K_LINUX_INIT                               IOCTL_ETHJPG2K(0x70)
/*! The size (in pages) of the buffer for the acquisition of images is returned. */
#define IOCTL_ETHJPG2K_LINUX_GETBSIZE                           IOCTL_ETHJPG2K(0x71)
/*! Sets the dimensions of the image ring buffer. Pass a ethjpg2k_linux_imgbuf
structure. */
#define IOCTL_ETHJPG2K_LINUX_SETBUF                             IOCTL_ETHJPG2K(0x72)

/* TW2835 */

/*! Writes a register of the TWA2835. The argument is a pointer to an
array of 2 ints: first is the register number, second is the value. */
#define IOCTL_ETHJPG2K_TW2835_WRITE_REG                        IOCTL_ETHJPG2K(0x80)
/*! Reads a register from the TWA2835. The argument is a pointer to an
array of 2 ints: first is the register number, second is the value. */
#define IOCTL_ETHJPG2K_TW2835_READ_REG                         IOCTL_ETHJPG2K(0x81)
/*! Resets the TWA2835. */
#define IOCTL_ETHJPG2K_TW2835_RESET                            IOCTL_ETHJPG2K(0x82)
/*! Sets the timeout (in seconds) when waiting for an event from the
video front-end. */
#define IOCTL_ETHJPG2K_TW2835_TOUT_SEC                         IOCTL_ETHJPG2K(0x83)
/*! Does a batch of operations to reduce the number of context switches
  (it'useful when downloading bitmaps for example). The argument is a
  pointer to an an array of ints. The first one is the length of
  remaining elements. After it the actual operations. */
#define IOCTL_ETHJPG2K_TW2835_BATCH                            IOCTL_ETHJPG2K(0x84)

/* ADV212 debugging */

/*! Writes a register of the ADV212. The argument is a pointer to an
array of 2 ints: first is the register number, second is the value. */
#define IOCTL_ETHJPG2K_ADV212_WRITE_REG                        IOCTL_ETHJPG2K(0x90)
/*! Reads a register from the ADV212. The argument is a pointer to an
array of 2 ints: first is the register number, second is the value. */
#define IOCTL_ETHJPG2K_ADV212_READ_REG                         IOCTL_ETHJPG2K(0x91)

/*Typedef's & struct's*/


/*! \brief Header for a encoded image

  This is the image header passed to the user level when encoding. */
struct ethjpg2k_linux_hdr {
  /*! 0xfffffff0 or 0xfffffff1 depending if frame is even or
    odd in an interlaced stream. */
  unsigned long field;
  /*! The current frame number. */
  unsigned long index;
  /*! Attributes of the frame (see ADV212 data
    sheet for more information) */
  union
  {
    unsigned long info;
    struct {
      /*!  Header version:
0x02 for encode firmware with App ID = 0xFF82
       */
      unsigned char version;
      unsigned char reserved;
      /*! VFORMAT parameter:
0 = NTSC 4:2:2
1 = PAL 4:2:2
2 = 1080i Luminance
3 = 1080i Chrominance
4 = Custom specific
5 = 720p/60 Luminance
6 = 720p/60 Chrominance
8 = NTSC de-interlaced
9 = PAL de-interlaced
       */
      unsigned char videoFormat;
      /*! Output code stream format  COD_STYLE
parameter setting:
bit 0 = ADV202 Raw Format
bit 1 = J2C Format
bit 2 = Jp2 Format
bit 3 = 1 = ADV202 header not generated
bit 3 = 0 = ADV202 header generated
bit 4 to 7 only valid with J2c or Jp2 format,
otherwise these bits must all be 0.
bit 4 = 1 = Include EPH
bit 4 = 0 = EPH not included
bit 5 = 1 = Include SOP
bit 5 = 0 = SOP not included
bit 6 = 1 = Include PLT
bit 6 = 0 = PLT not included
bit 7 = 1 = include PPT
bit 7 = 0 = Packet headers with Packet Body
       */
      unsigned char codeFormat;
    } Info_s;
  } Info_u;
  /*! Size (in bytes) of the acquired image. */
  unsigned long size;

  /*! Location in the mmaped area of the image. */
  unsigned long start;
  /*! Number of image lost since last read (if we
    don't keep up with the pace of image acquired). */
  unsigned int overflow;
  /*! Number that has to be written to the device when finished. */
  unsigned int ticket;
};

/*! \brief Header for the space for the image to be decoded

   This is the image header passed to the user level when decoding. */
struct ethjpg2k_linux_decode {
  /*! The starting address of free space in the buffer. */
  unsigned long start;
  /*! Amount of available bytes */
  unsigned long size;
};

/*! \brief ADV212 encoder parameters

  These are parameters that specifies the JPEG2K encoder
  parameters. For more information see the TechNote Getting Started
  with the ADV212 */
typedef struct _ETHJPG2K_VIDEO_ENCODE_FIRMWARE_PARAMS
{
	unsigned char 	VFORMAT;			/*!< Video Standard */
	unsigned char 	PREC;				/*!< Precision */
	unsigned char 	XFORMLEV;			/*!< Wavelet Transform Levels */
	unsigned char 	UNI;				/*!< Component Polarity */

	unsigned char 	CBSIZE;				/*!< Codeblock Dimensions */
	unsigned char 	WKERNEL_QUANT;		/*!< Wavelet Kernel */
	unsigned char 	STALLPAR;			/*!< SkipFields */
	unsigned char 	ATTRTYPE;			/*!< Attribute Output Format */

	unsigned char 	RCTYPE;			/*!< Rate Control */
	unsigned char 	RCVAL[3];				/*!< Target Size Or Quality */

	unsigned char 	J2KPROG;			/*!< J2K Progression */
	unsigned char 	PICFG;				/*!< Pixel If Config */
	unsigned char 	QFACT;				/*!< Quantize Factor */
	unsigned char 	COD_STYLE;			/*!< Code Output Format */

	unsigned short	STEPSIZES[19];		/*!< Quantize Stepsizes */
	unsigned char 	LOAD_SS;
	unsigned char	LOAD_VW;			/*!< Visual Weight Options */

	unsigned short 	VW_Y;				/*!< Y_Visual Weight Factor */
	unsigned short 	VW_CB;				/*!< Cb_VisualWeightFactor[19] */
	unsigned short 	VW_CR;				/*!< Cr_VisualWeightFactor[19] */
	unsigned char	RESERVED[6];
	unsigned long 	LayersTargets[16];	/*!< Target Size or Target Qualit */
} ETHJPG2K_VIDEO_ENCODE_FIRMWARE_PARAMS, *PETHJPG2K__VIDEO_ENCODE_FIRMWARE_PARAMS;

/*! \brief ADV212 decoder parameters

  These are parameters that specifies the JPEG2K decoder
  parameters. For more information see the TechNote Getting Started
  with the ADV212 */
typedef struct _ETHJPG2K__VIDEO_DECODE_FIRMWARE_PARAMS
{
	unsigned char 	VFORMAT;		/*!< Video Standard */
	unsigned char 	PREC;			/*!< Precision */
	unsigned char	RESERVED_0;
	unsigned char 	UNI;			/*!< Component Polarity */
	unsigned char	RESERVED_1[5];
	unsigned char 	PICFG;			/*!< Pixel If Config */
	unsigned char 	DRES;			/*!< Decode Resolution Settings */
	unsigned char 	COD_STYLE;		/*!< Code Input Format */
} ETHJPG2K_VIDEO_DECODE_FIRMWARE_PARAMS, PETHJPG2K_VIDEO_DECODE_FIRMWARE_PARAMS;


/*! \brief ADV212 HIPI mode parameters

  These are parameters that specifies the JPEG2K HIPI mode
  parameters. For more information see the TechNote Getting Started
  with the ADV212 */
typedef struct _ETHJPG2K_HIPI_FIRMWARE_PARAMS
{
	unsigned short	XTOT;	/*!<  number oy bytes per lin */
	unsigned short 	YTOT;   /*!<  number of lines per fram */
	unsigned char 	PMODE;  /*!<  pixel forma */
	union {
		ETHJPG2K_VIDEO_ENCODE_FIRMWARE_PARAMS	EncodeParams; /*!< parameters if encoding  */
		ETHJPG2K_VIDEO_DECODE_FIRMWARE_PARAMS	DecodeParams; /*!< parameters if decoding  */
	} sub;
} ETHJPG2K_HIPI_FIRMWARE_PARAMS, *PETHJPG2K_HIPI_FIRMWARE_PARAMS;


/*! \brief ADV212 generic parameters

  Union for passing parameters to the encoding/decoding/HIPI engine of the ADV212 */
typedef union _ETHJPG2K_FIRMWARE_PARAMS
{
	ETHJPG2K_VIDEO_ENCODE_FIRMWARE_PARAMS	EncodeParams;
	ETHJPG2K_VIDEO_DECODE_FIRMWARE_PARAMS	DecodeParams;
	ETHJPG2K_HIPI_FIRMWARE_PARAMS		HipiParams;

} ETHJPG2K_FIRMWARE_PARAMS, *PETHJPG2K_FIRMWARE_PARAMS;

/*! \brief ADV212 firmware and parameters descriptor

  The firmware and the parameters to load into the ADV212 */
typedef struct _ETHJPG2K_FIRMWARE_DATA
{
  	unsigned char	Type;	/*!< type: endoer/decoder/HIPI  */
	unsigned long*	pFirmware; /*!< pointer to the firmware bits  */
	unsigned long 	FirmwareSizeInDwords; /*!< size of the firmware */
	PETHJPG2K_FIRMWARE_PARAMS	pFirmwareParameters; /*!< parameters for the ADV212 */

} ETHJPG2K_FIRMWARE_DATA, *PETHJPG2K_FIRMWARE_DATA;

/*! \brief Image ring buffer description

  Used with IOCTL_ETHJPG2K_LINUX_SETBUF  */
struct ethjpg2k_linux_imgbuf {
  int img_max_size;		/*!< max dimension of an image */
  int img_n;			/*!< max number of images (at least 3) */
};


#endif  //ETHJPG2K_H
