/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/
#ifndef _PLX9065_REGISTERS_H_
#define _PLX9065_REGISTERS_H_ 1

#define PLX9056_VENID	0x10B5
#define PLX9056_DEVID	0x9056

#define SIZE_8BITS	1
#define SIZE_16BITS	2
#define SIZE_32BITS	4



/*
Local Configuration Registers
PLX_ADDR_* are registers offset, other #defines are bits
*/

#define  PLX_ADDR_LAS0BA                     0x04
#define  PLX_LADR0_ENABLE                    0x01

#define  PLX_ADDR_ENDIAN                     0x000C
#define  PLX_REGSTARTUP_ENDIAN               0x00C0   // DMA 0 and DMA 1 in big-endian mode for local side

#define  PLX_ADDR_LMISC2                     0x000F
#define  PLX_LMISC2_TIMEOUT_TA_ENA           0x0001
#define  PLX_LMISC2_TIMEOUT_TA_1024          0x0002



#define  PLX_ADDR_LBRD0                      0x0018
#define  PLX_REGSTARTUP_LBRD0            0x4A430143L  // [ 1: 0] =   11b  : Local address space 0 width: 32-bit bus
																		// [ 5: 2] = 0000b  : 0 WS
                                                      // [ 6]    =    1b  : TA/READY input enabled
                                                      // [ 7]    =    0b  : Local address space 0 burst disabled
                                                      // [ 8]    =    1b  : Local address space 0 prefetch disabled
                                                      // [ 9:26]          : Expansion ROM space stuff - doesn't matter for us
																		// [27]    =    1b  : The PCI 9056 de-asserts TRDY# when the Direct Slave Write FIFO is full.
                                                      // [31:28] = 0010b  : Number of PCI clocks (multiplied by 8) from the beginning of a
                                                      //                  : Direct Slave access until a PCI Retry is issued, if the transfer has
                                                      //                  : not completed. Valid for Read cycles only if MARBR[24]=0.
                                                      //                  : Valid for Write cycles only if LBRD0[27]=1.
                                                      //
#define  PLX_ADDR_DMA_MODE_ARB               0x00AC
#define  PLX_REGSTARTUP_DMA_MODE_ARB     0x0120000CL

#define  PCI_STS_MASTER_DATA_PERR       (0x00000001L <<  8)
#define  PCI_STS_TARGET_ABT_SIGNALLED   (0x00000001L << 11)
#define  PCI_STS_TARGET_ABT_RECEIVED    (0x00000001L << 12)
#define  PCI_STS_MASTER_ABT_RECEIVED    (0x00000001L << 13)

#define  PLX_ADDR_DMA0_MODE                  0x0080
#define  PLX_ADDR_DMA1_MODE                  0x0094
#define  PLX_DMA_MODE_LCLBUS08           0x00000000L
#define  PLX_DMA_MODE_LCLBUS16           0x00000001L
#define  PLX_DMA_MODE_LCLBUS32           0x00000003L
#define  PLX_DMA_MODE_WS_CNTR_MASK       0x0000003CL
#define  PLX_DMA_MODE_TA_RDY_ENA         0x00000040L
#define  PLX_DMA_MODE_CBURST_ENA         0x00000080L
#define  PLX_DMA_MODE_LBURST_ENA         0x00000100L
#define  PLX_DMA_MODE_SG_ENA             0x00000200L
#define  PLX_DMA_MODE_INT_DONE_ENA       0x00000400L
#define  PLX_DMA_MODE_CONST_LCL_ADDR     0x00000800L
#define  PLX_DMA_MODE_DEMAND             0x00001000L
#define  PLX_DMA_MODE_TERMINATE_FAST     0x00008000L
#define  PLX_DMA_MODE_CLEAR_COUNT        0x00010000L
#define  PLX_DMA_MODE_INT_PCI            0x00020000L

#define  PLX_ADDR_DMA0_PCI_ADDR              0x0084
#define  PLX_ADDR_DMA1_PCI_ADDR              0x0098

#define  PLX_ADDR_DMA0_LCL_ADDR              0x0088
#define  PLX_ADDR_DMA1_LCL_ADDR              0x009C

#define  PLX_ADDR_DMA0_XFR_SIZE              0x008C
#define  PLX_ADDR_DMA1_XFR_SIZE              0x00A0

#define  PLX_ADDR_DMA0_DESC_POINTER          0x0090
#define  PLX_ADDR_DMA1_DESC_POINTER          0x00A4

#define  PLX_DESC_PTR_LOCATION_PCI      0x000000001L
#define  PLX_DESC_PTR_END_OF_CHAIN      0x000000002L
#define  PLX_DESC_PTR_INT_TERM_COUNT    0x000000004L
#define  PLX_DESC_PTR_LCL_TO_PCI        0x000000008L
#define  PLX_DESC_NEXT_ADDR_MASK        0xFFFFFFFF0L
#define  PLX_DESC_ALIGN_SIZE                      32

#define  PLX_ADDR_DMA0_CMDSTS                0x00A8
#define  PLX_ADDR_DMA1_CMDSTS                0x00A9

#define  PLX_CMDSTS_ENABLE                    0x01
#define  PLX_CMDSTS_START                     0x02
#define  PLX_CMDSTS_ABORT                     0x04
#define  PLX_CMDSTS_CLRINT                    0x08
#define  PLX_CMDSTS_DONE                      0x10

#define  PLX_ADDR_INTCTRLSTS                 0x0068
#define	 PLX_INTCTRLSTS_PM_INT_ACTIVE   (0x00000001L <<  5)
#define	 PLX_INTCTRLSTS_PE_INT_ACTIVE   (0x00000001L <<  7)
#define  PLX_INTCTRLSTS_PCI_INT_ENA     (0x00000001L <<  8)
#define  PLX_INTCTRLSTS_ABORT_ENA       (0x00000001L << 10)
#define  PLX_INTCTRLSTS_LCL_INT_ENA     (0x00000001L << 11)
#define  PLX_INTCTRLSTS_RETRY_ABORT_ENA (0x00000001L << 12)
#define  PLX_INTCTRLSTS_ABORT_ACTIVE    (0x00000001L << 14)
#define  PLX_INTCTRLSTS_LCL_INT_ACTIVE  (0x00000001L << 15)
#define  PLX_INTCTRLSTS_DMA0_INT_ENA    (0x00000001L << 18)
#define  PLX_INTCTRLSTS_DMA1_INT_ENA    (0x00000001L << 19)
#define  PLX_INTCTRLSTS_DMA0_INT_ACTIVE (0x00000001L << 21)
#define  PLX_INTCTRLSTS_DMA1_INT_ACTIVE (0x00000001L << 22)
#define  PLX_INTCTRLSTS_DMA0_BMSTR_NABT (0x00000001L << 25)
#define  PLX_INTCTRLSTS_DMA1_BMSTR_NABT (0x00000001L << 26)
#define  PLX_INTCTRLSTS_RETRY_TGET_NABT (0x00000001L << 27)

#define  PLX_ADDR_RTCTRL              		0x006C
#define  PLX_RTCTRL_LOCAL_RST           (0x00000001L << 30)


/*Miscellaneous*/
#define	PLX_DMA0_INT	\
	(PLX_INTCTRLSTS_PCI_INT_ENA | PLX_INTCTRLSTS_DMA0_INT_ENA | \
	PLX_INTCTRLSTS_DMA0_INT_ACTIVE )
#define	PLX_DMA1_INT	\
	(PLX_INTCTRLSTS_PCI_INT_ENA | PLX_INTCTRLSTS_DMA1_INT_ENA | \
	PLX_INTCTRLSTS_DMA1_INT_ACTIVE )
#define	PLX_LOCAL_INT	\
	(PLX_INTCTRLSTS_PCI_INT_ENA | PLX_INTCTRLSTS_LCL_INT_ENA  |	\
	PLX_INTCTRLSTS_LCL_INT_ACTIVE )



#endif
