/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

//#define DEBUG 1

//#define DEBUG_IO 1

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/usb.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/fcntl.h>
#include <linux/seq_file.h>
#include <linux/cdev.h>
#include <linux/kthread.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/unaligned.h>
#include <asm/dma.h>

#include "plx9056.h"
#include "driver.h"

#include "adv202.h"
#include "plx9056_registers.h"


/* miscellanous buffers for ioctl */
static unsigned long firmware[32768/4];
static ETHJPG2K_FIRMWARE_PARAMS firmware_parameters;
static ETHJPG2K_FIRMWARE_DATA firmware_data;
static struct ethjpg2k_linux_imgbuf linux_imgbuf;

static int restart_adv202(struct plx_dma * pdev)
{
	int type = 0;
	unsigned short id = 0;
	int force = 0;
	PETHJPG2K pethjpg2k = pdev->plx->pethjpg2k;

	if (pdev->plx->adv_started)
		return 0;

	pdev->plx->adv_started = 1;

	switch(pdev->plx->mode) {
	case PLX_ENCODE: type = FW_VIDEO_ENCODE; id = 0xff82; break;
	case PLX_DECODE: type = FW_VIDEO_DECODE; id = 0xffa2; break;
	case PLX_HIPI_ENCODE: type = FW_HIPI_ENCODE; id = 0xff82; break;
	case PLX_HIPI_DECODE: type = FW_HIPI_DECODE; id = 0xffa2; break;
	case PLX_HIPI_LOOP: type = FW_HIPI_LOOP; id = 0; break;
	default:
		printq("bug @ %s %d\n", __FUNCTION__, __LINE__);
	}

	if (pdev->plx->mode == PLX_HIPI_ENCODE || pdev->plx->mode == PLX_HIPI_DECODE)
	  force = 0x0084;

	pr_debug("restarting ADV202\n");
	pethjpg2k->FirmwareType = type;
	ADV202_Reset(pethjpg2k);
	if (ADV202_Init(pethjpg2k, 0, force) != 1)
	  printk("Initializaion of ADV212 failed!\n");
	if (pdev->plx->mode != PLX_HIPI_LOOP) {
		ADV202_LoadFirmware(pethjpg2k, firmware, 32768/4);
		ADV202_LoadFirmwareParameters(pethjpg2k, type, &firmware_parameters);
	}
	ADV202_SetupDMA(pethjpg2k);

	if (pdev->plx->mode == PLX_HIPI_LOOP) {
		ADV202_WriteIndirect(pethjpg2k, 0xFFFF1418, 0x00100000);
		if (ADV202_ReadIndirect(pethjpg2k, 0xFFFF1418) != 0x00100000)
			printq("error on settingup LOOP mode!\n");
	}
	else {
		if (!ADV202_CheckApplicationID(pethjpg2k, id)) {
			printq("ID mishmash after dnload!\n");
			return -EBADF;
		}
		ADV202_StartExecution(pethjpg2k);
	}

	return 0;
}

static void init_board(struct plx_dma * pdev, int is_ntsc)
{
	PETHJPG2K pethjpg2k = pdev->plx->pethjpg2k;

	pethjpg2k->Flags.bNtsc = is_ntsc;
	ADV202_Reset(pethjpg2k);
	pr_debug("%s\n", __FUNCTION__);
}

void init_cpld(PETHJPG2K pDevice)
{
	unsigned long	Data;

	Data = RD( pDevice->PLX9056_BAR2+CPLD_STATUS );
	Data &= ~0xf;
	Data |= 3;
	WD( pDevice->PLX9056_BAR2+CPLD_STATUS, Data);
}

static int start_encdec(struct plx_dma * pdev)
{
	int i;
	pr_debug("%s %d %d\n", __FUNCTION__, __LINE__, pdev->opened & PLX_RUNNING);

	if (pdev->opened & PLX_RUNNING)
		return -EBUSY;

	pdev->head = pdev->tail = 0;
	for(i=0; i < pdev->img_n; i++)
		pdev->img[i].status = IMAGE_IDLE;

	restart_adv202(pdev);
	pdev->opened |= PLX_RUNNING;

	if ((pdev->opened & PLX_OPENED_IN) == PLX_OPENED_IN) {
		plx_irqon(pdev, PLX_INTCTRLSTS_DMA0_INT_ENA);
		pdev->thread = kthread_run(encode_thread, pdev, "ctr1474_enc%d", pdev->minor);
	}
	else if ((pdev->opened & PLX_OPENED_OUT) == PLX_OPENED_OUT) {
		plx_irqon(pdev, PLX_INTCTRLSTS_DMA1_INT_ENA);
		pdev->thread = kthread_run(decode_thread, pdev, "ctr1474_dec%d", pdev->minor);
	}
	else
		printk("BUG %s %d\n", __FUNCTION__, __LINE__);

	return 0;
}

void stop_encdec(struct plx_dma * pdev)
{
	pr_debug("%s %d\n", __FUNCTION__, __LINE__);
	pdev->opened &= ~PLX_RUNNING;
	pdev->plx->adv_started = 0;
	if (pdev->thread_running)
		kthread_stop(pdev->thread);
	if ((pdev->opened & PLX_OPENED_IN) == PLX_OPENED_IN) {
		plx_irqoff(pdev, PLX_INTCTRLSTS_DMA0_INT_ENA);
	}
	else if ((pdev->opened & PLX_OPENED_OUT) == PLX_OPENED_OUT) {
		plx_irqoff(pdev, PLX_INTCTRLSTS_DMA1_INT_ENA);
	}
	else
		printk("BUG %s %d\n", __FUNCTION__, __LINE__);
}

int plx_ioctl(struct inode *inode, struct file *filp,
                 unsigned int cmd, unsigned long arg)
{
	struct plx_dma *pdev = (struct plx_dma *) filp->private_data;
	int ret = -ENOTTY;
	int t;

	switch (cmd) {
	case IOCTL_ETHJPG2K_LINUX_INIT:
		init_board(pdev, (int) arg);
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_ENCDEC_LOAD_FIRMWARE:
		if (copy_from_user(&firmware_data, (unsigned char *) arg, sizeof(firmware_data)))
			return -EFAULT;

		switch (firmware_data.Type) {
		case FW_HIPI_ENCODE:
			pdev->plx->mode = PLX_HIPI_ENCODE;
			break;
		case FW_HIPI_DECODE:
			pdev->plx->mode = PLX_HIPI_DECODE;
			break;
		case FW_HIPI_LOOP:
			pdev->plx->mode = PLX_HIPI_LOOP;
			break;
		}

		if (pdev->plx->mode != PLX_HIPI_LOOP) {
			if (copy_from_user(firmware, firmware_data.pFirmware, 32768))
				return -EFAULT;
			if (copy_from_user(&firmware_parameters, firmware_data.pFirmwareParameters ,
					   sizeof(firmware_parameters)))
				return -EFAULT;
			if (pdev->plx->mode == PLX_ENCODE) {
				pdev->plx->skip = firmware_parameters.EncodeParams.STALLPAR;
				pdev->plx->is_ntsc = (firmware_parameters.EncodeParams.VFORMAT == 0 ||
						      firmware_parameters.EncodeParams.VFORMAT == 8) ? 1 : 0;
				pdev->plx->is_interlaced = (firmware_parameters.EncodeParams.VFORMAT == 8 ||
							    firmware_parameters.EncodeParams.VFORMAT == 9) ? 1 : 0;
				pdev->plx->coding_style = firmware_parameters.EncodeParams.COD_STYLE;
			}
			if (pdev->plx->mode == PLX_HIPI_DECODE) {
				pdev->plx->image_size = firmware_parameters.HipiParams.XTOT
					* firmware_parameters.HipiParams.YTOT;
			}
			if (pdev->plx->mode == PLX_HIPI_ENCODE) {
				pdev->plx->skip = firmware_parameters.HipiParams.sub.EncodeParams.STALLPAR;
				pdev->plx->is_ntsc = (firmware_parameters.HipiParams.sub.EncodeParams.VFORMAT == 0 ||
						      firmware_parameters.HipiParams.sub.EncodeParams.VFORMAT == 8) ? 1 : 0;
				pdev->plx->is_interlaced = ((firmware_parameters.HipiParams.sub.EncodeParams.VFORMAT == 8 ||
							     firmware_parameters.HipiParams.sub.EncodeParams.VFORMAT == 9) ?
							    1 : 0);
				pdev->plx->coding_style = firmware_parameters.HipiParams.sub.EncodeParams.COD_STYLE;
			}
		}
		else {
			pdev->plx->image_size = firmware_data.FirmwareSizeInDwords;
			/* note: misleading, no Dwords but bytes */
		}
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_ENCDEC_START:
		return start_encdec(pdev);
		break;

	case IOCTL_ETHJPG2K_ENCDEC_STOP:
		stop_encdec(pdev);
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_ENC_QUALITY:
		if (copy_from_user(&t, (unsigned int *) arg, sizeof(unsigned int)))
			return -EFAULT;
		if ((pdev->opened & PLX_RUNNING) == 0)
			return -EINVAL;
		ADV202_SetQ(pdev->plx->pethjpg2k, t);
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_LINUX_GETBSIZE: {
		unsigned long size = pdev->img_max_size * pdev->img_n / PAGE_SIZE;

		if (copy_to_user((unsigned long *) arg, &size, sizeof(unsigned long)))
			return -EFAULT;
		ret = 0;
	}
		break;

	case IOCTL_ETHJPG2K_LINUX_SETBUF:
		if (pdev->opened & PLX_RUNNING || pdev->is_mapped)
			return -EBUSY;
		if (copy_from_user(&linux_imgbuf, (struct ethjpg2k_linux_imgbuf *) arg,
				   sizeof(struct ethjpg2k_linux_imgbuf)))
			return -EFAULT;
		free_global_memory(pdev);
		pdev->img_n = linux_imgbuf.img_n;
		pdev->img_max_size = linux_imgbuf.img_max_size;
		alloc_global_memory(pdev);
		ret = 0;
		break;

	  /* GPIOs */

	case IOCTL_ETHJPG2K_GPIO_READ:
		t = RD(pdev->plx->pethjpg2k->PLX9056_BAR2 + GPIO_IN) & 0xFFL;
		if (copy_to_user((unsigned int *) arg, &t, sizeof(unsigned int)))
			return -EFAULT;
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_GPIO_WRITE:
		if (copy_from_user(&t, (unsigned int *) arg, sizeof(unsigned int)))
			return -EFAULT;
		WD(pdev->plx->pethjpg2k->PLX9056_BAR2 + GPIO_OUT, t);
		ret = 0;
	  break;

	default:
		printq("unknown ioctl %08x\n", cmd);
	}

	return ret;
}

unsigned long image_timeout(struct plx_dma *pdev)
{
	unsigned long ret;

	if (pdev->plx->mode == PLX_HIPI_ENCODE ||
	    pdev->plx->mode == PLX_HIPI_DECODE ||
	    pdev->plx->mode == PLX_HIPI_LOOP)
		return 5 * HZ;

	ret = (1000 + (25 *(1 + pdev->plx->skip) *
		      (pdev->plx->is_interlaced ? 2 : 1))) *
		HZ / 1000;
	if (ret == 0)
		ret = 1;
	return ret;
}
