/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

//#define DEBUG 1

//#define DEBUG_IO 1

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/usb.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/fcntl.h>
#include <linux/seq_file.h>
#include <linux/cdev.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/unaligned.h>
#include <asm/dma.h>

#include "plx9056.h"
#include "plx9056_registers.h"

#include "adv202.h"

void plx_irqon(struct plx_dma *dma, u32 Mask)
{
	unsigned long InterruptReg;
	struct plx_dev *bSlave = dma->plx;

	InterruptReg =  PLX9056_ReadConfigReg( bSlave, PLX_ADDR_INTCTRLSTS, SIZE_32BITS);
	PLX9056_WriteConfigReg( bSlave, PLX_ADDR_INTCTRLSTS, InterruptReg | Mask | (1<<8), SIZE_32BITS);
}

void plx_irqoff(struct plx_dma *dma, u32 Mask)
{
	unsigned long InterruptReg;
	struct plx_dev *bSlave = dma->plx;

	InterruptReg =  PLX9056_ReadConfigReg( bSlave, PLX_ADDR_INTCTRLSTS, SIZE_32BITS);
	PLX9056_WriteConfigReg( bSlave, PLX_ADDR_INTCTRLSTS, (InterruptReg & ~Mask) | (1<<8), SIZE_32BITS);
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
irqreturn_t plx_pci_isr(int irq, void *__data, struct pt_regs *r)
#else
irqreturn_t plx_pci_isr(int irq, void *__data)
#endif
{
	struct pci_dev *dev = (struct pci_dev *) __data;
	struct plx_dev *bSlave = pci_get_drvdata(dev);
	struct plx_dma * dma;
	unsigned long	PLX9056_IntStatus;
	unsigned char	PLX9056_DmaStatus;

	PLX9056_IntStatus = PLX9056_ReadConfigReg(bSlave, PLX_ADDR_INTCTRLSTS, SIZE_32BITS);

	if ( !(PLX9056_IntStatus &
	       (PLX_INTCTRLSTS_DMA0_INT_ACTIVE | PLX_INTCTRLSTS_DMA1_INT_ACTIVE | PLX_INTCTRLSTS_LCL_INT_ACTIVE))) {
		return IRQ_NONE;
	}

	pr_debug("%s: IRQ: %08lx\n", __FUNCTION__, PLX9056_IntStatus);

	/*DMA0 interrupt*/
	if ( (PLX9056_IntStatus & PLX_DMA0_INT) == PLX_DMA0_INT )
	{
		// disable IRQ
		PLX9056_WriteConfigReg(bSlave, PLX_ADDR_INTCTRLSTS,
			PLX9056_IntStatus & ~PLX_INTCTRLSTS_DMA0_INT_ENA,
			SIZE_32BITS);

		PLX9056_DmaStatus = (unsigned char)PLX9056_ReadConfigReg(bSlave,PLX_ADDR_DMA0_CMDSTS,SIZE_8BITS);
		PLX9056_WriteConfigReg(bSlave,PLX_ADDR_DMA0_CMDSTS,PLX9056_DmaStatus | PLX_CMDSTS_CLRINT,SIZE_8BITS);

		dma = &bSlave->dma[0];
		complete(&dma->dma_done);
	}

	/*DMA1 interrupt*/
	if ( (PLX9056_IntStatus & PLX_DMA1_INT) == PLX_DMA1_INT )
	{

		/*Disable, clear, re-enable*/
		PLX9056_WriteConfigReg(bSlave, PLX_ADDR_INTCTRLSTS,
			PLX9056_IntStatus & ~PLX_INTCTRLSTS_DMA1_INT_ENA,
			SIZE_32BITS);

		PLX9056_DmaStatus = (unsigned char)PLX9056_ReadConfigReg(bSlave,PLX_ADDR_DMA1_CMDSTS,SIZE_8BITS);
		PLX9056_WriteConfigReg(bSlave,PLX_ADDR_DMA1_CMDSTS,PLX9056_DmaStatus | PLX_CMDSTS_CLRINT,SIZE_8BITS);

		dma = &bSlave->dma[1];
		complete(&dma->dma_done);
	}

	/*ADV202 or motion detector interrupt */
	if ( (PLX9056_IntStatus & PLX_LOCAL_INT ) == PLX_LOCAL_INT )
	{
		pr_debug("LOCAL interrupt\n");
		plx_irqoff(&bSlave->dma[0], PLX_INTCTRLSTS_LCL_INT_ENA);
		bSlave->twa_got_irq = 1;
		wake_up_interruptible(&bSlave->twa_wait);
		return IRQ_HANDLED;
	}

	// reenable IRQ
	PLX9056_WriteConfigReg(bSlave, PLX_ADDR_INTCTRLSTS, PLX9056_IntStatus, SIZE_32BITS);

	return IRQ_HANDLED;
}

int plx_request_irq(struct pci_dev *dev, void *isr_data)
{
	struct plx_dev *pdev = pci_get_drvdata(dev);
	int result = 0;

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,21)
	result= request_irq(dev->irq, plx_pci_isr,
			    IRQF_SHARED, pdev->name, isr_data);
#else
	result= request_irq(dev->irq, plx_pci_isr,
			    SA_SHIRQ, pdev->name, isr_data);
#endif
	dev_dbg(&dev->dev, "irq %d got %d\n", dev->irq, result);

	return result;
}

void plx_free_irq(struct pci_dev *dev, void *isr_data)
{
	free_irq(dev->irq,isr_data);
}

