/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

#ifndef _PLX9056_H_
#define _PLX9056_H_ 1

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>

#include "ethjpg2k.h"
#include "adv202_hdr.h"
#include "driver.h"

struct plx_dma_desc {
	unsigned long	BufferPhysicalAddress;
	unsigned long	BufferLocalAddress;
	unsigned long	BufferTransferSize;
	unsigned long	NextDescriptorPhysicalAddress;
};

struct plx_dev;
struct plx_dma;

struct plx_image {
	int status;
#define IMAGE_IDLE 0
#define IMAGE_DMA 1
#define IMAGE_READY 2
#define IMAGE_USER 3

	struct ethjpg2k_linux_hdr hdr; /* for encoding */
	struct ethjpg2k_linux_decode dec; /* for decoding */
};

struct plx_dma {
	/* generic */

	volatile int opened;
#define PLX_CLOSED 0
#define PLX_OPENED_IN 1
#define PLX_OPENED_OUT 2
#define PLX_RUNNING 4
	struct plx_dev *plx;
	int minor;
	struct cdev cdev;

	/* buffer */

	struct plx_dma_desc *pg_dma;                /* DMA's page table address*/
	dma_addr_t pg_dma_phys;      /* DMA's page table physical address*/

	/* buffer page management */
	u8 *vbuffer;
	volatile u32 head;
	volatile u32 tail;

	/* images buffers */
	struct plx_image * img;
	int img_n;
	int img_max_size;
	int is_mapped;

	/* both encoding and decoding */
	wait_queue_head_t pready; /* for waking up read/write */
	wait_queue_head_t pready1; /* for waking on decode data av. */
	struct task_struct *thread; /* encoding/decoding thread */
	int thread_running;
	struct completion dma_done; /* for signaling done DMA */

	/* overflow indicator */
	int overflow;
};

struct plx_dev {
	char name[16];
	struct pci_dev *pci;
	int mode;
#define PLX_NONE 0
#define PLX_ENCODE 1
#define PLX_DECODE 2
#define PLX_HIPI_ENCODE 3
#define PLX_HIPI_DECODE 4
#define PLX_HIPI_LOOP 5

	void *baseaddress;
	u32 pci_mem_phy0;
	u32 length;

	void *iobase;
	u32 pci_io_base;
	u32 iolength;

	struct plx_dma dma[2];
	/*
	   note: dma[0] is always input (adv202 encodes and send to
	   CPU), dma[1] is output (adv202 decodes and gets data from
	   CPU)
	*/

	/* parameters */

	int skip;		/* how many frames we skip in HW */
	int is_ntsc;		/* the norm for frame rate calculation */
	int is_interlaced;	/* if we do de-interlation */
	int coding_style;
	int image_size;		/* for HIPI */
	int adv_started;

	PETHJPG2K pethjpg2k;	/* compatibiity with windows ce code */

	/* twa2835 */
	struct cdev cdev;	/* for twa2835 */
	wait_queue_head_t twa_wait; /* twa2835 interrupt */
	int twa_got_irq;
};



extern int quiet;
#define printq(arg...) \
do { \
if (quiet) \
pr_debug(arg); \
else \
printk(arg); \
} while (0)


/* function prototypes */
void plx_irqon(struct plx_dma *dma, u32 Mask);
void plx_irqoff(struct plx_dma *dma, u32 Mask);
int plx_request_irq(struct pci_dev *dev, void *isr_data);
void plx_free_irq(struct pci_dev *dev, void *isr_data);
void plx_reg_write32(struct pci_dev *dev,u32 reg,u32 data);
u32 plx_reg_read32(struct pci_dev *dev,u32 reg);
void PLX9056_WriteConfigReg ( struct plx_dev * pdev, u32 reg, u32 val, int mode);
u32 PLX9056_ReadConfigReg ( struct plx_dev * pdev, u32 reg, int mode);
int dma_unmap(struct plx_dma *dma, u8 *buf, int size, int from_device);
int dma_map(struct plx_dma *dma, u8 *buf, int size, int from_device);
void start_dma(struct plx_dma *dma, int ch);
void stop_dma(struct plx_dma *dma, int DMAChannel);
int plx_ioctl(struct inode *inode, struct file *filp,
	      unsigned int cmd, unsigned long arg);
unsigned long image_timeout(struct plx_dma *pdev);
int encode_thread(void *pdev_);
int decode_thread(void *pdev_);
void stop_encdec(struct plx_dma * pdev);
int alloc_global_memory(struct plx_dma *dma);
void free_global_memory(struct plx_dma *dma);

#endif
