/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

//#define DEBUG 1

//#define DEBUG_IO 1

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/usb.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/fcntl.h>
#include <linux/seq_file.h>
#include <linux/cdev.h>
#include <linux/kthread.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/unaligned.h>
#include <asm/dma.h>

#include "plx9056.h"
#include "plx9056_registers.h"

#include "adv202.h"

int encode_thread(void *pdev_)
{
	struct plx_dma *pdev = pdev_;
	struct plx_image *img;
	unsigned long timeout = image_timeout(pdev);
	u8 *buf;
	ADV202_COMPRESSED_HEADER *hdr;
	int ret;
	PETHJPG2K pethjpg2k = pdev->plx->pethjpg2k;

	pdev->thread_running = 1;
	while(!kthread_should_stop()) {
		img = &pdev->img[pdev->head];
		BUG_ON(img->status != IMAGE_IDLE && img->status != IMAGE_READY);
		img->status = IMAGE_DMA;
		buf = &pdev->vbuffer[pdev->head * pdev->img_max_size];
		if (pdev->plx->mode == PLX_HIPI_DECODE || pdev->plx->mode == PLX_HIPI_LOOP) {
			img->hdr.field = 0xffffffff;
			img->hdr.index = 0;
			img->hdr.Info_u.info = 0;
			img->hdr.size = pdev->plx->image_size/4;
			pr_debug("forced image %08lx-%ld-%08lx-%ld to slot %d\n", img->hdr.field,
				 img->hdr.index, img->hdr.Info_u.info, img->hdr.size, pdev->head);
		}
		else {
			hdr = (ADV202_COMPRESSED_HEADER *) buf;
			dma_map(pdev, buf, 16, 1);
			start_dma(pdev, 0);
			ret = wait_for_completion_timeout(&pdev->dma_done, timeout);
			dma_unmap(pdev, buf, 16, 1);
			if (ret == 0) {
				stop_dma(pdev, 0);
				printq("timeout waiting for image hdr\n");
				img->status = IMAGE_IDLE;
				break;
			}
			img->hdr.field = htonl(hdr->Field);
			img->hdr.index = htonl(hdr->Index);
			img->hdr.Info_u.info = htonl(hdr->Info_u.Info);
			img->hdr.size = htonl(hdr->Size);
			pr_debug("got image %08lx-%ld-%08lx-%ld to slot %d\n", img->hdr.field,
				 img->hdr.index, img->hdr.Info_u.info, img->hdr.size, pdev->head);
		}
		if (!(pdev->plx->mode == PLX_HIPI_DECODE || pdev->plx->mode == PLX_HIPI_LOOP) &&
		    img->hdr.field != 0xfffffff0 && img->hdr.field != 0xfffffff1) {
			printq("Bad image header %08lx %08lx %08lx %08lx\n",
			       img->hdr.field, img->hdr.index, img->hdr.Info_u.info,
			       img->hdr.size);
			img->status = IMAGE_IDLE;
			break;
		}
		if (img->hdr.size * 4 > pdev->img_max_size) {
			printq("Image too big! Increase buffer size.\n");
			img->status = IMAGE_IDLE;
			break;
		}
		dma_map(pdev, buf, img->hdr.size * 4, 1);
		start_dma(pdev, 0);
		ret = wait_for_completion_timeout(&pdev->dma_done, timeout);
		dma_unmap(pdev, buf, img->hdr.size * 4, 1);
		if (ret == 0) {
			stop_dma(pdev, 0);
			printq("timeout waiting for image data\n");
			img->status = IMAGE_IDLE;
			break;
		}
		img->status = IMAGE_READY;
		if ((pdev->head + 1) % pdev->img_n == pdev->tail) {
			pr_debug("image ring overflow\n");
			pdev->overflow += 1;
		}
		else
			pdev->head = (pdev->head + 1) % pdev->img_n;
		wake_up_interruptible(&pdev->pready);
	}
	ADV202_Stop(pethjpg2k);
	pdev->thread_running = 0;
	return 0;
}

int decode_thread(void *pdev_)
{
	struct plx_dma *pdev = pdev_;
	struct plx_image *img;
	unsigned long timeout = image_timeout(pdev);
	u8 *buf;
	int ret;
	PETHJPG2K pethjpg2k = pdev->plx->pethjpg2k;

	pdev->thread_running = 1;
	while(!kthread_should_stop()) {
		if (pdev->head == pdev->tail) {
			ret = wait_event_interruptible_timeout(pdev->pready1,
							       (pdev->head != pdev->tail),
							       HZ / 10);
			if (ret == 0) {
				continue;
			}
			else if (ret == -ERESTARTSYS)
				continue;
		}
		pr_debug("sending slot %d @ %lu\n", pdev->tail, jiffies);
		img = &pdev->img[pdev->tail];
		BUG_ON(img->status != IMAGE_READY);
		img->status = IMAGE_DMA;
		buf = &pdev->vbuffer[pdev->tail * pdev->img_max_size];
		dma_map(pdev, buf, img->dec.size, 0);
		start_dma(pdev, 1);
		ret = wait_for_completion_timeout(&pdev->dma_done, timeout);
		dma_unmap(pdev, buf, img->dec.size, 0);
		if (ret == 0) {
			stop_dma(pdev, 1);
			printq("timeout waiting for image sent\n");
		}
		img->status = IMAGE_IDLE;
		pdev->tail = (pdev->tail + 1) % pdev->img_n;
		wake_up_interruptible(&pdev->pready);
		msleep(5);
	}
	ADV202_Stop(pethjpg2k);
	pdev->thread_running = 0;
	return 0;
}
