/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

//#define DEBUG 1

#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19)
#include <linux/config.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/smp_lock.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/usb.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/types.h>
#include <linux/proc_fs.h>
#include <linux/fcntl.h>
#include <linux/seq_file.h>
#include <linux/cdev.h>
#include <linux/workqueue.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/unaligned.h>
#include <asm/dma.h>

#include "plx9056.h"
#include "plx9056_registers.h"
#include "adv202.h"
#include "driver.h"
#include "twa2835_linux.h"

static int twa2835_tout_s = 10;
static int bb[1023];

static ssize_t twa2835_read(struct file *filp, char __user *buf, size_t count,
                loff_t *f_pos)
{
	struct plx_dev *pdev = (struct plx_dev *) filp->private_data;
	int ret = 1, t;

	pdev->twa_got_irq = 0;
	plx_irqon(&pdev->dma[0], PLX_INTCTRLSTS_LCL_INT_ENA);
	t = RD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS );
	WD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS, t | CPLD_TWA2835_IRQ);

	if (wait_event_interruptible_timeout(pdev->twa_wait, pdev->twa_got_irq, HZ * twa2835_tout_s))
	  ret = -ERESTARTSYS;
	if (pdev->twa_got_irq == 0)
	  ret = -ETIMEDOUT;

	plx_irqoff(&pdev->dma[0], PLX_INTCTRLSTS_LCL_INT_ENA);
	t = RD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS );
	WD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS, t & ~CPLD_TWA2835_IRQ);

	return ret;
}

static int twa2835_ioctl(struct inode *inode, struct file *filp,
                 unsigned int cmd, unsigned long arg)
{
	struct plx_dev *pdev = (struct plx_dev *) filp->private_data;
	unsigned int t[2], i;
	int ret = -ENOTTY;

	switch(cmd) {
	case IOCTL_ETHJPG2K_TW2835_WRITE_REG:
		if (copy_from_user(t, (unsigned int *) arg, 2*sizeof(unsigned int)))
			return -EFAULT;
		WD( pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_ADDR, t[0]);
		pr_debug("%s %d writting %08x to %08lx + %08x\n", __FUNCTION__, __LINE__, t[0],
			 pdev->pethjpg2k->PLX9056_BAR2, CPLD_TWA2835_ADDR);
		WD( pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_DATA, t[1]);
		pr_debug("%s %d writting %08x to %08lx + %08x\n", __FUNCTION__, __LINE__, t[1],
			 pdev->pethjpg2k->PLX9056_BAR2, CPLD_TWA2835_DATA);
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_TW2835_READ_REG:
		if (copy_from_user(t, (unsigned int *) arg, 2*sizeof(unsigned int)))
			return -EFAULT;
		pr_debug("%s %d writing %08x to %08lx + %08x\n", __FUNCTION__, __LINE__,
			 t[0], pdev->pethjpg2k->PLX9056_BAR2, CPLD_TWA2835_ADDR);
		WD( pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_ADDR, t[0]);
		t[1] = RD(pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_DATA) & 0xFFL;
		pr_debug("%s %d got %08x from %08lx + %08x\n", __FUNCTION__, __LINE__, t[1],
			 pdev->pethjpg2k->PLX9056_BAR2, CPLD_TWA2835_DATA);
		if (copy_to_user((unsigned int *) arg, t, 2*sizeof(unsigned int)))
			return -EFAULT;
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_TW2835_RESET:
		t[0] = RD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS );
		WD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS, t[0] & ~CPLD_TWA2835_RESET);
		Sleep(100);
		WD( pdev->pethjpg2k->PLX9056_BAR2+CPLD_STATUS, t[0] | CPLD_TWA2835_RESET);
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_TW2835_TOUT_SEC:
		if (copy_from_user(t, (unsigned int *) arg, 1 * sizeof(unsigned int)))
			return -EFAULT;
		twa2835_tout_s = t[0];
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_TW2835_BATCH:
		if (copy_from_user(t, (unsigned int *) arg, 1 * sizeof(unsigned int)))
			return -EFAULT;
		if (t[0] > 1023)
			return -EINVAL;
		if (copy_from_user(bb, &((unsigned int *) arg)[1], t[0] * sizeof(unsigned int)))
			return -EFAULT;
		for(i=0; i < t[0]; i++) {
			int cmd,addr,val,v;
			unsigned long deadline;

			cmd =  (bb[i] >> 24) & 0xff;
			addr = (bb[i] >> 8)  & 0xffff;
			val  = (bb[i] >> 0)  & 0xff;
			switch(cmd) {
			case 1:	/* WRITE */
				WD( pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_ADDR, addr);
				WD( pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_DATA, val);
				break;
			case 2:	/* POLL WAIT */
				deadline = jiffies + HZ;
				v = 0;
				while (time_before(jiffies, deadline)) {
					WD( pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_ADDR, addr);
					v = RD(pdev->pethjpg2k->PLX9056_BAR2 + CPLD_TWA2835_DATA) & 0xFFL;
					if ((v & val) == 0)
						break;
				}
				if (v & val)
					return -EIO;
				break;
			default:
				return -EINVAL;
			}
		}
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_GPIO_READ:
		t[0] = RD(pdev->pethjpg2k->PLX9056_BAR2 + GPIO_IN) & 0xFFL;
		if (copy_to_user((unsigned int *) arg, t, sizeof(unsigned int)))
			return -EFAULT;
		ret = 0;
		break;

	case IOCTL_ETHJPG2K_GPIO_WRITE:
		if (copy_from_user(t, (unsigned int *) arg, sizeof(unsigned int)))
			return -EFAULT;
		WD(pdev->pethjpg2k->PLX9056_BAR2 + GPIO_OUT, t[0]);
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_GPIO_WRITE_READ:
		t[0] = RD(pdev->pethjpg2k->PLX9056_BAR2 + GPIO_OUT);
		if (copy_to_user((unsigned int *) arg, t, sizeof(unsigned int)))
			return -EFAULT;
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_ADV212_WRITE_REG:
		if (copy_from_user(t, (unsigned int *) arg, 2*sizeof(unsigned int)))
			return -EFAULT;
		if (t[0] < 64)
		  ADV202_WriteDirect(pdev->pethjpg2k, t[0], t[1]);
		else
		  ADV202_WriteIndirect(pdev->pethjpg2k, t[0], t[1]);
		ret = 0;
		break;
	case IOCTL_ETHJPG2K_ADV212_READ_REG:
		if (copy_from_user(t, (unsigned int *) arg, 2*sizeof(unsigned int)))
			return -EFAULT;
		if (t[0] < 64)
		  t[1] = ADV202_ReadDirect(pdev->pethjpg2k, t[0]);
		else
		  t[1] = ADV202_ReadIndirect(pdev->pethjpg2k, t[0]);
		if (copy_to_user((unsigned int *) arg, t, 2*sizeof(unsigned int)))
			return -EFAULT;
		ret = 0;
		break;
	}

	return ret;
}

static int twa2835_open(struct inode *inode, struct file *filp)
{
	struct plx_dev *pdev;

	pdev = container_of(inode->i_cdev, struct plx_dev, cdev);
	pr_debug("%s\n", __FUNCTION__);
	filp->private_data = pdev;
	return 0;
}


struct file_operations twa2835_fops = {
	.owner =    THIS_MODULE,
	.read =     twa2835_read,
	.ioctl =    twa2835_ioctl,
	.open =     twa2835_open,
};
