/*
  Copyright 2007 Eurotech S.P.A. (Luca Calligaris)
  Copyright 2007 Christian Pellegrin <chripell@gmail.com> EVOL S.R.L.

  This software is released under the GPL v2. Please see the file
  COPYING in the top-level directory.
*/

#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <netinet/in.h>

#include "version.h"
#include "libctr1474.h"

#define MAX_PATH 255

ETHJPG2K_FIRMWARE_DATA fwdata;
ETHJPG2K_FIRMWARE_PARAMS fwparams;

int main(int argc, char *argv[]) {
  struct ctr1474_ch *enc;
  struct ctr1474_ch *dec;
  int mode, pmode, xtot, ytot, i;
  char *fw;
  int *ins, *outs;
  char **in, **out;

  printf("Eurotech CTR1474 utility, version " VERSION " compiled on " __DATE__ " " __TIME__ "\n");

  if (argc < 9) {
    printf("Syntax:\n");
    printf("test_loop [enc dev] [dec dev] [ctr dev] [firmware] [mode: 0=loop, 1=encode 2=decode] [pmode] [xtot] [ytot] [images ...]\n");
    exit(1);
  }

  enc = simple_ctr1474(argv[1], argv[3]);
  assert(enc);
  dec = simple_ctr1474(argv[2], argv[3]);
  assert(dec);
  fw = argv[4];
  mode = strtol(argv[5], NULL, 0);
  pmode = strtol(argv[6], NULL, 0);
  xtot = strtol(argv[7], NULL, 0);
  ytot = strtol(argv[8], NULL, 0);
  argc -= 9;
  argv += 9;

  ins = calloc(argc, sizeof(int)); assert(ins+1); /* 1 image since hipi is one frame behind */
  outs = calloc(argc, sizeof(int)); assert(outs);
  in = calloc(argc, sizeof(char *)); assert(in+1);
  out = calloc(argc, sizeof(char *)); assert(out);

  for(i=0; i<argc; i++) {
    char *fname = argv[i];
    FILE *f;
    int padded;

    f = fopen(fname, "r");
    if (!f) {
      fprintf(stderr, "Cannot open %s!\n", fname);
      exit(1);
    }
    fseek(f, 0, SEEK_END);
    ins[i] = ftell(f);
    assert(ins[i]);
    printf("image %d size %d\n", i, ins[i]);
    rewind(f);
    padded = ins[i];
    in[i] = calloc(1, padded);
    assert(in[i]);
    fread(in[i], ins[i], 1, f);
    ins[i] = padded;
    fclose(f);
  }
  in[i] = in[i-1];
  ins[i] = ins[i-1];

  memset(&fwdata, 0, sizeof(fwdata));
  memset(&fwparams, 0, sizeof(fwparams));
  switch(mode) {
  case 0:
    fwdata.Type = FW_HIPI_LOOP;
    fwdata.FirmwareSizeInDwords = xtot * ytot;
    break;
  case 1:
    fwdata.Type = FW_HIPI_ENCODE;
    fwparams.HipiParams.XTOT = xtot;
    fwparams.HipiParams.YTOT = ytot;
    fwparams.HipiParams.PMODE = pmode;
    fwparams.HipiParams.sub.EncodeParams.VFORMAT = 4;
    fwparams.HipiParams.sub.EncodeParams.XFORMLEV = 5;
    fwparams.HipiParams.sub.EncodeParams.UNI = 3;
    fwparams.HipiParams.sub.EncodeParams.CBSIZE = 1;
    fwparams.HipiParams.sub.EncodeParams.RCTYPE = 2;
    fwparams.HipiParams.sub.EncodeParams.RCVAL[1] = 1;
    fwparams.HipiParams.sub.EncodeParams.COD_STYLE = 2;
    break;
  case 2:
    fwdata.Type = FW_HIPI_DECODE;
    fwparams.HipiParams.XTOT = xtot;
    fwparams.HipiParams.YTOT = ytot;
    fwparams.HipiParams.PMODE = pmode;
    fwparams.HipiParams.sub.DecodeParams.VFORMAT = 4;
    fwparams.HipiParams.sub.DecodeParams.UNI = 3;
    fwparams.HipiParams.sub.DecodeParams.COD_STYLE = 1;
    break;
  default:
    assert(0);
  }

  simple_ctr1474_hipi(enc, dec,
		    fw, &fwdata, &fwparams,
		    argc,
		    in, ins,
		    out, outs);

  for(i=0; i<argc; i++) {
    char fname[MAX_PATH];
    FILE *f;

    sprintf(fname, "%s.jp2kout", argv[i]);
    f = fopen(fname, "w");
    if (!f) {
      fprintf(stderr, "Cannot open %s for writting!\n", fname);
      exit(1);
    }
    fwrite(out[i], outs[i], 1, f);
    printf("out image %d size %d\n", i, outs[i]);
    fclose(f);
  }

  return 0;
}
